"""
swCreateExplodeStepError_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCreateExplodeStepError_e.html

Status: 🟢
"""

from enum import IntEnum


class SWCreateExplodeStepErrorE(IntEnum):
    """Defines return codes when creating an explode step in SolidWorks."""

    SW_CREATE_EXPLODE_STEP_ERROR_SUCCESSFUL = 0  # Step created successfully
    SW_CREATE_EXPLODE_STEP_ERROR_GENERIC = 1  # Explode step creation failed
    SW_CREATE_EXPLODE_STEP_ERROR_NO_EXPLODE_VIEW = 2  # No explode view active
    SW_CREATE_EXPLODE_STEP_ERROR_NO_COMPONENTS = 3  # No components selected
    SW_CREATE_EXPLODE_STEP_ERROR_INVALID_RADIAL_AXIS = 4  # Invalid radial explode axis
    SW_CREATE_EXPLODE_STEP_ERROR_OPEN_EXPLODE_PMP = 5  # Explode PropertyManager is open
    SW_CREATE_EXPLODE_STEP_ERROR_EDITING_COMPONENT_IN_CONTEXT = 6  # Editing a component in context is blocking creation



"""
ISwAddin Interface Members

Reference:
https://help.solidworks.com/2024/english/api/swpublishedapi/solidworks.interop.swpublished~solidworks.interop.swpublished.iswaddin_members.html

Status: 🟢
"""

from pythoncom import VT_DISPATCH
from pythoncom import VT_I4
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.sldworks.interfaces.i_sldworks import ISldWorks


class ISwAddin(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"ISwAddin({self.com_object})"

    def connect_to_sw(self, this_sw: ISldWorks, cookie: int) -> bool:
        """
        Calls this method when the add-in is loaded.

        Reference:
        https://help.solidworks.com/2024/english/api/swpublishedapi/SolidWorks.Interop.swpublished~SolidWorks.Interop.swpublished.ISwAddin~ConnectToSW.html
        """
        in_this_sw = VARIANT(VT_DISPATCH, this_sw)
        in_cookie = VARIANT(VT_I4, cookie)

        com_object = self.com_object.ConnectToSW(in_this_sw, in_cookie)
        return bool(com_object)

    def disconnect_from_sw(self) -> bool:
        """
        Calls this method when SOLIDWORKS is about to be destroyed.

        Reference:
        https://help.solidworks.com/2024/english/api/swpublishedapi/SolidWorks.Interop.swpublished~SolidWorks.Interop.swpublished.ISwAddin~DisconnectFromSW.html
        """
        com_object = self.com_object.DisconnectFromSW
        return bool(com_object)



"""
swCustomInfoAddResult_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCustomInfoAddResult_e.html

Status: 🟢
"""

from enum import IntEnum


class SWCustomInfoAddResultE(IntEnum):
    """Result codes when adding custom properties."""

    SW_CUSTOM_INFO_ADD_RESULT_ADDED_OR_CHANGED = 0  # Success
    SW_CUSTOM_INFO_ADD_RESULT_GENERIC_FAIL = 1  # Failed to add the custom property
    SW_CUSTOM_INFO_ADD_RESULT_MISMATCH_EXISTING_TYPE = (
        2  # Existing property with same name has different type
    )
    SW_CUSTOM_INFO_ADD_RESULT_MISMATCH_SPECIFIED_TYPE = (
        3  # Value doesn't match the specified type
    )



"""
PYSWX EXCEPTIONS
"""

import sys
import traceback

from pyswx.logger import log


class BaseError(Exception):
    """Base class for all app exceptions. Logs to pyswx logger."""

    def __init__(self, msg: str) -> None:

        super().__init__(msg)
        if traceback.extract_tb(sys.exc_info()[2]):
            log.logger.exception(msg)
        else:
            log.logger.error(msg)


class DocumentError(BaseError): ...


class ArgumentError(BaseError): ...



"""
swSaveAsOptions_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swSaveAsOptions_e.html

Status: 🟢
"""

from enum import IntEnum


class SWSaveAsOptionsE(IntEnum):
    SW_SAVE_AS_OPTIONS_SILENT = 1
    SW_SAVE_AS_OPTIONS_COPY = 2  # Save the document as a copy and continue editing
    SW_SAVE_AS_OPTIONS_SAVE_REFERENCED = 4  # Save all referenced components
    SW_SAVE_AS_OPTIONS_AVOID_REBUILD_ON_SAVE = 8
    SW_SAVE_AS_OPTIONS_UPDATE_INACTIVE_VIEWS = 16  # Only applies to multi-sheet drawings
    SW_SAVE_AS_OPTIONS_OVERRIDE_SAVE_EMODEL = 32  # Overrides the Save eDrawings data setting
    SW_SAVE_AS_OPTIONS_IGNORE_BIOGRAPHY = 256  # Prune revision history to current file name
    SW_SAVE_AS_OPTIONS_COPY_AND_OPEN = 512  # Save as copy and open
    SW_SAVE_AS_OPTIONS_INCLUDE_VIRTUAL_SUB_ASM_COMPS = 1024  # Save components in virtual subassemblies
    SW_SAVE_AS_OPTIONS_EXPORT_TO_2D_PDF_FROM_INSPECTION = 2048  # Export drawing sheets from Inspection to 2D PDF



"""
swAssemblyLoadComponents_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swAssemblyLoadComponents_e.html

Status: 🟢
"""

from enum import IntEnum


class SWAssemblyLoadComponentsE(IntEnum):
    """Assembly loading options in Tools > System Options > Performance."""

    SW_ASSEMBLY_LOAD_COMPONENTS_AUTO_LOAD = 0  # Auto load components
    SW_ASSEMBLY_LOAD_COMPONENTS_MANUAL_LOAD = 1  # Manual load components



"""
swCloseReopenError_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCloseReopenError_e.html

Status: 🟢
"""

from enum import IntEnum


class SWCloseReopenErrorE(IntEnum):
    """Close and reopen errors."""

    SW_CLOSE_REOPEN_NO_ERROR = 0  # No error
    SW_CLOSE_REOPEN_UNKNOWN_ERROR = 1  # Unknown error
    SW_CLOSE_REOPEN_NO_INPUT_DOC_ERROR = 2  # Input document is null
    SW_CLOSE_REOPEN_OUTPUT_DOC_POINTER_ERROR = 3  # Output document is null
    SW_CLOSE_REOPEN_INVALID_DOC_ERROR = 4  # Document is not a drawing
    SW_CLOSE_REOPEN_CLOSE_DOC_ERROR = 5  # Error occurred during close
    SW_CLOSE_REOPEN_LOAD_GENERIC_ERROR = 6  # Error occurred during reopen
    SW_CLOSE_REOPEN_LOAD_FILE_NOT_FOUND_ERROR = 7  # File path specified does not exist
    SW_CLOSE_REOPEN_LOAD_INVALID_FILE_TYPE_ERROR = 8  # File type is not valid
    SW_CLOSE_REOPEN_LOAD_FUTURE_VERSION_ERROR = 9  # File to reopen is a future version
    SW_CLOSE_REOPEN_LOAD_SAME_TITLE_ALREADY_OPEN_ERROR = (
        10  # File with same title is already open
    )
    SW_CLOSE_REOPEN_LOAD_LIQUID_MACHINE_DOC_ERROR = 11  # LiquidMachine document error
    SW_CLOSE_REOPEN_MODIFIED_ERROR = 12  # Cannot close due to unsaved changes
    SW_CLOSE_REOPEN_LOAD_FILE_PATH_EMPTY_ERROR = 13  # File path is empty
    SW_CLOSE_REOPEN_LOAD_FILE_PATH_NON_DRAWING_ERROR = 14  # File is not a drawing



"""
swFeatMgrPane_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swFeatMgrPane_e.html

Status: 🟢
"""

from enum import IntEnum


class SWFeatMgrPaneE(IntEnum):
    """Available panes and states."""

    SW_FEAT_MGR_PANE_TOP = 0  # Top pane
    SW_FEAT_MGR_PANE_BOTTOM = 1  # Bottom pane
    SW_FEAT_MGR_PANE_TOP_HIDDEN = 2  # Top pane hidden
    SW_FEAT_MGR_PANE_BOTTOM_HIDDEN = 3  # Bottom pane hidden
    SW_FEAT_MGR_PANE_FLYOUT = 4  # Flyout pane



"""
SolidWorks.Interop.swconst Namespace

Reference:
https://help.solidworks.com/2024/english/api/swconst/SolidWorks.Interop.swconst~SolidWorks.Interop.swconst_namespace.html
"""

from pyswx.api.swconst.enumerations.sw_3d_experience_cfg_type_e import SW3DExperienceCfgTypeE
from pyswx.api.swconst.enumerations.sw_activate_doc_error_e import SWDocActivateErrorE
from pyswx.api.swconst.enumerations.sw_add_to_recent_document_list_e import SWAddToRecentDocumentListE
from pyswx.api.swconst.enumerations.sw_app_notify_e import SWAppNotifyE
from pyswx.api.swconst.enumerations.sw_application_type_e import SWApplicationTypeE
from pyswx.api.swconst.enumerations.sw_assembly_load_components_e import SWAssemblyLoadComponentsE
from pyswx.api.swconst.enumerations.sw_body_type_e import SWBodyTypeE
from pyswx.api.swconst.enumerations.sw_bom_configuration_anchor_type_e import SWBOMConfigurationAnchorTypeE
from pyswx.api.swconst.enumerations.sw_cad_family_cfg_options_e import SWCADFamilyCfgOptionsE
from pyswx.api.swconst.enumerations.sw_child_component_in_bom_option_e import SWChildComponentInBOMOptionE
from pyswx.api.swconst.enumerations.sw_close_reopen_error_e import SWCloseReopenErrorE
from pyswx.api.swconst.enumerations.sw_close_reopen_option_e import SWCloseReopenOptionE
from pyswx.api.swconst.enumerations.sw_component_solving_option_e import SWComponentSolvingOptionE
from pyswx.api.swconst.enumerations.sw_component_suppression_state_e import SWComponentSuppressionStateE
from pyswx.api.swconst.enumerations.sw_config_tree_sort_type_e import SWConfigTreeSortTypeE
from pyswx.api.swconst.enumerations.sw_configuration_options2_e import SWConfigurationOptions2E
from pyswx.api.swconst.enumerations.sw_configuration_type_e import SWConfigurationTypeE
from pyswx.api.swconst.enumerations.sw_create_explode_step_error_e import SWCreateExplodeStepErrorE
from pyswx.api.swconst.enumerations.sw_custom_info_add_result_e import SWCustomInfoAddResultE
from pyswx.api.swconst.enumerations.sw_custom_info_delete_result_e import SWCustomInfoDeleteResultE
from pyswx.api.swconst.enumerations.sw_custom_info_get_result_e import SWCustomInfoGetResultE
from pyswx.api.swconst.enumerations.sw_custom_info_set_result_e import SWCustomInfoSetResultE
from pyswx.api.swconst.enumerations.sw_custom_info_type_e import SWCustomInfoTypeE
from pyswx.api.swconst.enumerations.sw_custom_link_set_result_e import SWCustomLinkSetResultE
from pyswx.api.swconst.enumerations.sw_custom_property_add_option_e import SWCustomPropertyAddOptionE
from pyswx.api.swconst.enumerations.sw_document_types_e import SWDocumentTypesE
from pyswx.api.swconst.enumerations.sw_dwg_paper_sizes_e import SWDwgPaperSizesE
from pyswx.api.swconst.enumerations.sw_export_data_file_type_e import SWExportDataFileType_e
from pyswx.api.swconst.enumerations.sw_export_data_sheets_to_export_e import SWExportDataSheetsToExportE
from pyswx.api.swconst.enumerations.sw_feat_mgr_pane_e import SWFeatMgrPaneE
from pyswx.api.swconst.enumerations.sw_file_load_error_e import SWFileLoadErrorE
from pyswx.api.swconst.enumerations.sw_file_load_warning_e import SWFileLoadWarningE
from pyswx.api.swconst.enumerations.sw_file_save_error_e import SWFileSaveErrorE
from pyswx.api.swconst.enumerations.sw_file_save_warning_e import SWFileSaveWarningE
from pyswx.api.swconst.enumerations.sw_geometry_to_save_e import SWGeometryToSaveE
from pyswx.api.swconst.enumerations.sw_in_configuration_opts_e import SWInConfigurationOptsE
from pyswx.api.swconst.enumerations.sw_message_box_btn_e import SWMessageBoxBtnE
from pyswx.api.swconst.enumerations.sw_message_box_icon_e import SWMessageBoxIconE
from pyswx.api.swconst.enumerations.sw_message_box_result_e import SWMessageBoxResultE
from pyswx.api.swconst.enumerations.sw_open_doc_options_e import SWOpenDocOptionsE
from pyswx.api.swconst.enumerations.sw_rebuild_on_activation_e import SWRebuildOnActivationOptionsE
from pyswx.api.swconst.enumerations.sw_save_as_options_e import SWSaveAsOptionsE
from pyswx.api.swconst.enumerations.sw_save_as_version_e import SWSaveAsVersionE
from pyswx.api.swconst.enumerations.sw_save_items_path_error_e import SWSaveItemsPathErrorE
from pyswx.api.swconst.enumerations.sw_select_type_e import SWSelectTypeE
from pyswx.api.swconst.enumerations.sw_task_pane_tab_e import SWTaskPaneTabE
from pyswx.api.swconst.enumerations.sw_user_preference_string_value_e import SWUserPreferenceStringValueE



"""
IExportPdfData Interface Members

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IExportPdfData_members.html

Status: 🔴
"""

from typing import List

from pythoncom import VT_ARRAY
from pythoncom import VT_BSTR
from pythoncom import VT_I4
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.swconst.enumerations import SWExportDataSheetsToExportE


class IExportPdfData(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"IExportPdfData({self.com_object})"

    @property
    def export_as_3d(self) -> bool:
        """
        Gets or sets whether to export this part or drawing document to 3D PDF.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IExportPdfData~ExportAs3D.html
        """
        return bool(self.com_object.ExportAs3D)

    @export_as_3d.setter
    def export_as_3d(self, value: bool) -> None:
        """True to export this part or drawing document to 3D PDF, false to not"""
        self.com_object.ExportAs3D = value

    @property
    def view_pdf_after_saving(self) -> bool:
        """
        Gets or sets whether to view the PDF file to which a part or drawing is saved.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IExportPdfData~ViewPdfAfterSaving.html
        """
        return bool(self.com_object.ViewPdfAfterSaving)

    @view_pdf_after_saving.setter
    def view_pdf_after_saving(self, value: bool) -> None:
        """True to view the PDF file, false to not"""
        self.com_object.ViewPdfAfterSaving = value

    def get_sheets(self) -> List[str]:
        """
        Gets the names of the sheets to export.

        Remarks:
        Call IModelDocExtension::SaveAs after calling this method.

        Returns:
        Array of the names of the sheets to export

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IExportPdfData~GetSheets.html
        """
        com_object = self.com_object.GetSheets
        return [sheet for sheet in com_object] if com_object else []

    def get_which_sheets(self) -> SWExportDataSheetsToExportE:
        """
        Gets the drawing sheets to export to PDF.

        Returns:
        Drawing sheets to export to PDF as defined in swExportDataSheetsToExport_e

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IExportPdfData~GetWhichSheets.html
        """
        com_object = self.com_object.GetWhichSheets
        return SWExportDataSheetsToExportE(com_object)

    def set_sheets(self, which: SWExportDataSheetsToExportE, sheets: List[str]) -> bool:
        """
        Sets the drawing sheets to export.

        Args:
            which (SWExportDataSheetsToExportE): Drawing sheets to export to PDF as defined in swExportDataSheetsToExport_e
            sheets (List[str]): Array of the names of the drawing sheets to export

        Returns:
            True if the drawings sheets are set to export to PDF, false if not

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IExportPdfData~SetSheets.html
        """
        in_which = VARIANT(VT_I4, which.value)
        in_sheets = VARIANT(VT_BSTR | VT_ARRAY, [str(sheet) for sheet in sheets])

        com_object = self.com_object.SetSheets(in_which, in_sheets)

        return bool(com_object)



"""
swCustomInfoGetResult_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCustomInfoGetResult_e.html

Status: 🟢
"""

from enum import IntEnum


class SWCustomInfoGetResultE(IntEnum):
    """Result codes when getting custom properties."""

    SW_CUSTOM_INFO_GET_RESULT_CACHED_VALUE = 0  # Cached value was returned
    SW_CUSTOM_INFO_GET_RESULT_NOT_PRESENT = 1  # Custom property does not exist
    SW_CUSTOM_INFO_GET_RESULT_RESOLVED_VALUE = 2  # Resolved value was returned



"""
ASSEMBLY TOOLS // OPEN ASSEMBLY
"""

from pathlib import Path
from typing import Tuple

from pyswx.api.sldworks.interfaces.i_assembly_doc import IAssemblyDoc
from pyswx.api.sldworks.interfaces.i_document_specification import IDocumentSpecification
from pyswx.api.sldworks.interfaces.i_model_doc_2 import IModelDoc2
from pyswx.api.sldworks.interfaces.i_sldworks import ISldWorks
from pyswx.api.swconst.enumerations import SWDocumentTypesE
from pyswx.api.swconst.enumerations import SWRebuildOnActivationOptionsE
from pyswx.exceptions import DocumentError


def open_assembly(
    swx: ISldWorks,
    assembly_path: Path,
    document_specification: IDocumentSpecification | None = None,
) -> Tuple[IModelDoc2, IAssemblyDoc]:
    """
    Open a SolidWorks assembly model.

    Args:
        swx (ISldWorks): The SolidWorks application instance.
        assembly_path (Path): The path to the SolidWorks assembly file to be exported.
        document_specification (IDocumentSpecification | None): The document specification to use. Defaults to None.

    Returns:
        Tuple[IModelDoc2, IPartDoc]: A tuple containing the model document and assembly document.

    Remarks:
        The following document specification options are set:
        - document_type: SWDocumentTypesE.SW_DOC_ASSEMBLY
        - use_light_weight_default: True
        - light_weight: True
        - silent: True
        - ignore_hidden_components: True
        - view_only: False

    Raises:
        DocumentError: Raised if there is an error opening the document.
        ValueError: Raised if no active document is found.
        ValueError: Raised if the active document is not an assembly.
    """
    assembly_open_spec = swx.get_open_doc_spec(file_name=assembly_path)
    assembly_open_spec.document_type = SWDocumentTypesE.SW_DOC_ASSEMBLY
    assembly_open_spec.use_light_weight_default = True
    assembly_open_spec.light_weight = True
    assembly_open_spec.silent = True
    assembly_open_spec.ignore_hidden_components = True
    assembly_open_spec.view_only = False
    assembly_model = swx.open_doc7(specification=document_specification or assembly_open_spec)

    if assembly_open_spec.warning is not None:
        swx.logger.warning(assembly_open_spec.warning.name)

    if assembly_open_spec.error is not None:
        swx.logger.error(assembly_open_spec.error.name)
        raise DocumentError(assembly_open_spec.error.name)

    if assembly_model is None:
        raise ValueError("No active document found")

    model_type = assembly_model.get_type()
    if model_type != SWDocumentTypesE.SW_DOC_ASSEMBLY:
        raise ValueError(f"Active document is not an assembly: {model_type.name}")

    assembly_model = swx.activate_doc_3(
        name=assembly_model.get_path_name(),
        use_user_preferences=False,
        option=SWRebuildOnActivationOptionsE.SW_REBUILD_ACTIVE_DOC,
    )

    return assembly_model, IAssemblyDoc(assembly_model.com_object)



"""
swExportDataFileType_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swExportDataFileType_e.html

Status: 🟢
"""

from enum import IntEnum


class SWExportDataFileType_e(IntEnum):
    """Export data file types."""

    SW_EXPORT_PDF_DATA = 1



"""
swFileSaveWarning_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swFileSaveWarning_e.html

Status: 🟢
"""

from enum import IntEnum


class SWFileSaveWarningE(IntEnum):
    SW_FILE_SAVE_WARNING_UNKNOWN = 0
    SW_FILE_SAVE_WARNING_REBUILD_ERROR = 1
    SW_FILE_SAVE_WARNING_NEEDS_REBUILD = 2
    SW_FILE_SAVE_WARNING_VIEWS_NEED_UPDATE = 4
    SW_FILE_SAVE_WARNING_ANIMATOR_NEED_TO_SOLVE = 8
    SW_FILE_SAVE_WARNING_ANIMATOR_FEATURE_EDITS = 16
    SW_FILE_SAVE_WARNING_EDRWINGS_BAD_SELECTION = 32
    SW_FILE_SAVE_WARNING_ANIMATOR_LIGHT_EDITS = 64
    SW_FILE_SAVE_WARNING_ANIMATOR_CAMERA_VIEWS = 128
    SW_FILE_SAVE_WARNING_ANIMATOR_SECTION_VIEWS = 256
    SW_FILE_SAVE_WARNING_MISSING_OLE_OBJECTS = 512
    SW_FILE_SAVE_WARNING_OPENED_VIEW_ONLY = 1024
    SW_FILE_SAVE_WARNING_XML_INVALID = 2048



"""
swCustomPropertyAddOption_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCustomPropertyAddOption_e.html

Status: 🟢
"""

from enum import IntEnum


class SWCustomPropertyAddOptionE(IntEnum):
    """Options when adding custom properties."""

    SW_CUSTOM_PROPERTY_ONLY_IF_NEW = 0  # Add only if the property is new
    SW_CUSTOM_PROPERTY_DELETE_AND_ADD = 1  # Delete existing and add new property
    SW_CUSTOM_PROPERTY_REPLACE_VALUE = 2  # Replace value of existing property



"""
swBOMConfigurationAnchorType_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SolidWorks.Interop.swconst~SolidWorks.Interop.swconst.swBOMConfigurationAnchorType_e.html

Status: 🟢
"""

from enum import IntEnum


class SWBOMConfigurationAnchorTypeE(IntEnum):
    """BOM table configuration anchor types."""

    SW_BOM_CONFIGURATION_ANCHOR_TOP_LEFT = 1  # Upper-left corner
    SW_BOM_CONFIGURATION_ANCHOR_TOP_RIGHT = 2  # Upper-right corner
    SW_BOM_CONFIGURATION_ANCHOR_BOTTOM_LEFT = 3  # Lower-left corner
    SW_BOM_CONFIGURATION_ANCHOR_BOTTOM_RIGHT = 4  # Lower-right corner



"""
swConfigurationType_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SolidWorks.Interop.swconst~SolidWorks.Interop.swconst.swConfigurationType_e.html

Status: 🟢
"""

from enum import IntEnum


class SWConfigurationTypeE(IntEnum):
    """Types of configurations."""

    SW_CONFIGURATION_STANDARD = 0  # Standard configuration
    SW_CONFIGURATION_ASMACHINED = 1  # As Machined configuration
    SW_CONFIGURATION_ASWELDED = 2  # As Welded configuration
    SW_CONFIGURATION_SHEETMETAL = 3  # Sheet Metal configuration
    SW_CONFIGURATION_SPEEDPAK = 4  # SpeedPak configuration



"""
swFeatMgrPane_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swFeatMgrPane_e.html

Status: 🟢
"""

from enum import Enum


class swFeatMgrPane_e(Enum):
    """
    Available panes and states for the Feature Manager.
    """

    swFeatMgrPaneTop = 0
    swFeatMgrPaneBottom = 1
    swFeatMgrPaneTopHidden = 2
    swFeatMgrPaneBottomHidden = 3
    swFeatMgrPaneFlyout = 4



"""
swRebuildOnActivation_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swRebuildOnActivation_e.html

Status: 🟢
"""

from enum import IntEnum


class SWRebuildOnActivationOptionsE(IntEnum):
    SW_USER_DECISION = 0  # Prompt the user whether to rebuild the activated document
    SW_DONT_REBUILD_ACTIVE_DOC = 1  # Do not rebuild the activated document
    SW_REBUILD_ACTIVE_DOC = 2  # Rebuild the activated document



"""
swApplicationType_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swApplicationType_e.html

Status: 🟢
"""

from enum import IntEnum


class SWApplicationTypeE(IntEnum):
    """Types of SOLIDWORKS application."""

    SW_APPLICATION_TYPE_DESKTOP = 0  # Desktop SOLIDWORKS
    SW_APPLICATION_TYPE_3DEXPERIENCE = 1  # 3DEXPERIENCE SOLIDWORKS
    SW_APPLICATION_TYPE_WITH_CONNECTOR = 2  # SOLIDWORKS with 3DEXPERIENCE connector



"""
IDrawingDoc Interface Members

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IDrawingDoc_members.html

Status: 🔴
"""

from pathlib import Path

from pythoncom import VT_BOOL
from pythoncom import VT_BSTR
from pythoncom import VT_BYREF
from pythoncom import VT_DISPATCH
from pythoncom import VT_I4
from pythoncom import VT_R8
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.sldworks.interfaces.i_view import IView


class IDrawingDoc(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"IDrawingDoc({self.com_object})"

    def create_draw_view_from_model_view_3(
        self, model_name: Path, view_name: str, loc_x: float, loc_y: float, loc_z: float
    ) -> IView:
        """
        Creates a drawing view on the current drawing sheet using the specified model view.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.idrawingdoc~createdrawviewfrommodelview3.html
        """
        in_model_name = VARIANT(VT_BSTR, str(model_name))
        in_view_name = VARIANT(VT_BSTR, view_name)
        in_loc_x = VARIANT(VT_R8, loc_x)
        in_loc_y = VARIANT(VT_R8, loc_y)
        in_loc_z = VARIANT(VT_R8, loc_z)

        com_object = self.com_object.CreateDrawViewFromModelView3(
            in_model_name, in_view_name, in_loc_x, in_loc_y, in_loc_z
        )
        return IView(com_object)



"""
IAdvancedSaveAsOptions Interface Members

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions_members.html

Status: 🟢
"""

from pathlib import Path
from typing import List
from typing import Tuple

from pythoncom import VT_ARRAY
from pythoncom import VT_BOOL
from pythoncom import VT_BSTR
from pythoncom import VT_BYREF
from pythoncom import VT_I4
from pythoncom import VT_VARIANT
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.swconst.enumerations import SWGeometryToSaveE
from pyswx.api.swconst.enumerations import SWSaveItemsPathErrorE

type IdsList = List[int]
type NamesList = List[str]
type PathsList = List[Path]


class IAdvancedSaveAsOptions(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"IAdvancedSaveAsOptions({self.com_object})"

    @property
    def configuration_to_save(self) -> None:
        """
        Write Only. Getter not implemented.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~ConfigurationsToSave.html
        """
        raise NotImplementedError("Write Only. Getter not implemented.")

    @configuration_to_save.setter
    def configuration_to_save(self, value: List["IAdvancedSaveAsOptions"]) -> None:
        """
        Array of configurations to save.
        """
        self.com_object.ConfigurationsToSave = VARIANT(
            VT_BSTR | VT_ARRAY, [v.com_object for v in value]
        )

    @property
    def description(self) -> None:
        """
        Write Only. Getter not implemented.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~Description.html
        """
        raise NotImplementedError("Write Only. Getter not implemented.")

    @description.setter
    def description(self, value: str) -> None:
        """
        Adds a desciption of the save.
        """
        self.com_object.Description = VARIANT(VT_BSTR, value)

    @property
    def geometry_to_save(self) -> None:
        """
        Write Only. Getter not implemented.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~GeometryToSave.html
        """
        raise NotImplementedError("Write Only. Getter not implemented.")

    @geometry_to_save.setter
    def geometry_to_save(self, value: SWGeometryToSaveE) -> None:
        self.com_object.GeometryToSave = VARIANT(VT_I4, value.value)

    @property
    def override_defaults(self) -> None:
        """
        Write Only. Getter not implemented.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~OverrideDefaults.html
        """
        raise NotImplementedError("Write Only. Getter not implemented.")

    @override_defaults.setter
    def override_defaults(self, value: bool) -> None:
        self.com_object.OverrideDefaults = VARIANT(VT_BOOL, value)

    @property
    def preserve_geometry_references(self) -> None:
        """
        Write Only. Getter not implemented.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~PreserveGeometryReferences.html
        """
        raise NotImplementedError("Write Only. Getter not implemented.")

    @preserve_geometry_references.setter
    def preserve_geometry_references(self, value: bool) -> None:
        self.com_object.PreserveGeometryReferences = VARIANT(VT_BOOL, value)

    @property
    def save_all_as_copy(self) -> None:
        """
        Write Only. Getter not implemented.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~SaveAllAsCopy.html
        """
        raise NotImplementedError("Write Only. Getter not implemented.")

    @save_all_as_copy.setter
    def save_all_as_copy(self, value: bool) -> None:
        self.com_object.SaveAllAsCopy = VARIANT(VT_BOOL, value)

    @property
    def save_as_previous_version(self) -> None:
        """
        Write Only. Getter not implemented.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~SaveAsPreviousVersion.html
        """
        raise NotImplementedError("Write Only. Getter not implemented.")

    @save_as_previous_version.setter
    def save_as_previous_version(self, value: int) -> None:
        self.com_object.SaveAllAsCopy = VARIANT(VT_I4, value)

    def get_items_name_and_path(self) -> Tuple[IdsList, NamesList, PathsList]:
        """
        Gets all reference components' names and paths.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~GetItemsNameAndPath.html
        """
        out_ids_list = VARIANT(VT_VARIANT | VT_BYREF, [])
        out_names_list = VARIANT(VT_VARIANT | VT_BYREF, [])
        out_paths_list = VARIANT(VT_VARIANT | VT_BYREF, [])

        self.com_object.GetItemsNameAndPath(
            out_ids_list, out_names_list, out_paths_list
        )

        return (
            [int(i) for i in out_ids_list.value],
            [str(i) for i in out_names_list.value],
            [Path(i) for i in out_paths_list.value],
        )

    def modify_items_name_and_path(
        self, ids_list: IdsList, names_list: NamesList, paths_list: PathsList
    ) -> SWSaveItemsPathErrorE:
        """
        Modifies the specified reference components with the specified names and paths.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~ModifyItemsNameAndPath.html
        """

        in_ids_list = VARIANT(VT_I4 | VT_ARRAY, [int(i) for i in ids_list])
        in_names_list = VARIANT(VT_BSTR | VT_ARRAY, [str(i) for i in names_list])
        in_paths_list = VARIANT(VT_BSTR | VT_ARRAY, [str(i) for i in paths_list])

        com_object = self.com_object.ModifyItemsNameAndPath(
            in_ids_list, in_names_list, in_paths_list
        )

        return SWSaveItemsPathErrorE(com_object)

    def set_prefix_suffix_to_all(self, prefix_string: str, suffix_string: str) -> None:
        """
        Sets a prefix and/or a suffix on all component reference names.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IAdvancedSaveAsOptions~SetPrefixSuffixToAll.html
        """

        in_prefix_string = VARIANT(VT_BSTR, prefix_string)
        in_suffix_string = VARIANT(VT_BSTR, suffix_string)

        self.com_object.SetPrefixSuffixToAll(in_prefix_string, in_suffix_string)



"""
IFrame Interface Members

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame_members.html

Status: 🟢
"""

from pathlib import Path
from typing import List

from pythoncom import VT_ARRAY
from pythoncom import VT_BSTR
from pythoncom import VT_BYREF
from pythoncom import VT_DISPATCH
from pythoncom import VT_I4
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.sldworks.interfaces.i_model_window import IModelWindow
from pyswx.api.sldworks.interfaces.i_status_bar_pane import IStatusBarPane
from pyswx.api.swconst.enumerations import SWDocumentTypesE
from pyswx.api.swconst.enumerations import SWSelectTypeE


class IFrame(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"IFrame({self.com_object})"

    @property
    def keep_invisible(self) -> bool:
        """
        Gets or sets whether to keep the SOLIDWORKS frame invisible.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~KeepInvisible.html
        """
        return bool(self.com_object.KeepInvisible)

    @keep_invisible.setter
    def keep_invisible(self, value: bool) -> None:
        self.com_object.KeepInvisible = value

    @property
    def menu_pinned(self) -> bool:
        """
        Gets or sets whether the SOLIDWORKS main menu is pinned.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~MenuPinned.html
        """
        return bool(self.com_object.MenuPinned)

    @menu_pinned.setter
    def menu_pinned(self, value: bool) -> None:
        self.com_object.MenuPinned = value

    @property
    def model_windows(self) -> bool:
        """
        Gets the client model windows for this frame.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~ModelWindows.html
        """
        return bool(self.com_object.ModelWindows)

    @model_windows.setter
    def model_windows(self, value: bool) -> None:
        self.com_object.ModelWindows = value

    def add_menu(self, menu: str, position: int) -> bool:
        """
        Adds a menu item.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~AddMenu.html
        """
        in_menu = VARIANT(VT_BSTR, menu)
        in_position = VARIANT(VT_I4, position)

        com_object = self.com_object.AddMenu(in_menu, in_position)
        return bool(com_object)

    def add_menu_item2(
        self,
        menu: str,
        item: str,
        position: int,
        callback_func_and_module: str,
        bitmap_file_name: Path,
    ) -> bool:
        """
        Adds a menu item and bitmap or a separator to an existing menu.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~AddMenuItem2.html
        """
        in_menu = VARIANT(VT_BSTR, menu)
        in_item = VARIANT(VT_BSTR, item)
        in_position = VARIANT(VT_I4, position)
        in_callback_func_and_module = VARIANT(VT_BSTR, callback_func_and_module)
        in_bitmap_file_name = VARIANT(VT_BSTR, str(bitmap_file_name))

        com_object = self.com_object.AddMenuItem2(
            in_menu,
            in_item,
            in_position,
            in_callback_func_and_module,
            in_bitmap_file_name,
        )
        return bool(com_object)

    def add_menu_popup_icon(
        self,
        doc_type: SWDocumentTypesE,
        select_type: SWSelectTypeE,
        hint_string: str,
        callback_func_and_module: str,
        custom_names: str,
        bitmap_file_path: str,
    ) -> bool:
        """
        Adds an icon to a context-sensitive menu of a C++ SOLIDWORKS add-in.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~AddMenuPopupIcon.html
        """
        in_doc_type = VARIANT(VT_I4, doc_type.value)
        in_select_type = VARIANT(VT_I4, select_type.value)
        in_hint_string = VARIANT(VT_BSTR, hint_string)
        in_callback_func_and_module = VARIANT(VT_BSTR, callback_func_and_module)
        in_custom_names = VARIANT(VT_BSTR, custom_names)
        in_bitmap_file_path = VARIANT(VT_BSTR, str(bitmap_file_path))

        com_object = self.com_object.AddMenuPopupIcon(
            in_doc_type,
            in_select_type,
            in_hint_string,
            in_callback_func_and_module,
            in_custom_names,
            in_bitmap_file_path,
        )
        return bool(com_object)

    def add_menu_popup_icon3(
        self,
        doc_type: SWDocumentTypesE,
        select_type: SWSelectTypeE,
        hint_string: str,
        identifier: int,
        callback_function: str,
        callback_update_function: str,
        custom_names: str,
        image_list: List[Path],
    ) -> bool:
        """
        Adds an icon to a context-sensitive menu of a SOLIDWORKS add-in.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~AddMenuPopupIcon3.html
        """
        in_doc_type = VARIANT(VT_I4, doc_type.value)
        in_select_type = VARIANT(VT_I4, select_type.value)
        in_hint_string = VARIANT(VT_BSTR, hint_string)
        in_identifier = VARIANT(VT_I4, identifier)
        in_callback_function = VARIANT(VT_BSTR, callback_function)
        in_callback_update_function = VARIANT(VT_BSTR, callback_update_function)
        in_custom_names = VARIANT(VT_BSTR, custom_names)
        in_image_list = VARIANT(VT_BSTR | VT_ARRAY, [str(i) for i in image_list])

        com_object = self.com_object.AddMenuPopupIcon3(
            in_doc_type,
            in_select_type,
            in_hint_string,
            in_identifier,
            in_callback_function,
            in_callback_update_function,
            in_custom_names,
            in_image_list,
        )
        return bool(com_object)

    def add_menu_popup_item2(
        self,
        doc_type: SWDocumentTypesE,
        select_type: SWSelectTypeE,
        item: str,
        callback_func_and_module: str,
        custom_names: str,
        unused: int,
        bitmap_file_name: Path,
    ) -> bool:
        """
        Adds a menu item or separator to a shortcut menu (i.e., a right-click pop-up menu).

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~AddMenuPopupItem2.html
        """
        in_doc_type = VARIANT(VT_I4, doc_type.value)
        in_select_type = VARIANT(VT_I4, select_type.value)
        in_item = VARIANT(VT_BSTR, item)
        in_callback_func_and_module = VARIANT(VT_BSTR, callback_func_and_module)
        in_custom_names = VARIANT(VT_BSTR, custom_names)
        in_unused = VARIANT(VT_I4, unused)
        in_bitmap_file_name = VARIANT(VT_BSTR, str(bitmap_file_name))

        com_object = self.com_object.AddMenuPopupItem2(
            in_doc_type,
            in_select_type,
            in_item,
            in_callback_func_and_module,
            in_custom_names,
            in_unused,
            in_bitmap_file_name,
        )
        return bool(com_object)

    def get_hwnd(self) -> int:
        """
        Gets the window handle for the main frame.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~GetHWnd.html
        """
        return int(self.com_object.GetHWnd)

    def get_hwnd_x64(self) -> int:
        """
        Gets the window handle for the main frame in 64-bit applications.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~GetHWndx64.html
        """
        return int(self.com_object.GetHWndx64)

    def get_menu(self) -> int:
        """
        Gets the menu handle for the main frame.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~GetMenu.html
        """
        return int(self.com_object.GetMenu)

    def get_menu_x64(self) -> int:
        """
        Gets the menu handle for the main frame in 64-bit applications.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~GetMenux64.html
        """
        return int(self.com_object.GetMenux64)

    def get_model_window_count(self) -> int:
        """
        Gets the number of child model windows for this frame.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~GetModelWindowCount.html
        """
        return int(self.com_object.GetModelWindowCount)

    def get_status_bar_pane(self) -> IStatusBarPane:
        """
        Gets a pointer to one of up to five panes on the right side of the status bar.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~GetStatusBarPane.html
        """
        return IStatusBarPane(self.com_object.GetStatusBarPane)

    def get_sub_menu_count(self, doc_type: SWDocumentTypesE, full_menu_name: str) -> int:
        """
        Gets the number of submenus for this frame.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~GetSubMenuCount.html
        """
        in_doc_type = VARIANT(VT_I4, doc_type.value)
        in_full_menu_name = VARIANT(VT_BSTR, full_menu_name)

        return int(self.com_object.GetSubMenuCount(in_doc_type, in_full_menu_name))

    def get_sub_menus(self, doc_type: SWDocumentTypesE, full_menu_name: str) -> List[str]:
        """
        Gets the submenus for this frame.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~GetSubMenuCount.html
        """
        in_doc_type = VARIANT(VT_I4, doc_type.value)
        in_full_menu_name = VARIANT(VT_BSTR, full_menu_name)

        return [str(i) for i in self.com_object.GetSubMenus(in_doc_type, in_full_menu_name)]

    def remove_menu(self, menu_item_string: str) -> None:
        """
        Removes a menu item.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~RemoveMenu.html
        """
        in_menu_item_string = VARIANT(VT_BSTR, menu_item_string)
        self.com_object.RemoveMenu(in_menu_item_string)

    def remove_menu_popup_icon(self, index: int, doc_type: SWDocumentTypesE, select_type: SWSelectTypeE) -> bool:
        """
        Removes an icon from a context-sensitive shortcut (popup) menu.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~RemoveMenuPopupIcon.html
        """
        in_index = VARIANT(VT_I4, index)
        in_doc_type = VARIANT(VT_I4, doc_type.value)
        in_select_type = VARIANT(VT_I4, select_type.value)

        com_object = self.com_object.RemoveMenuPopupIcon(in_index, in_doc_type, in_select_type)
        return bool(com_object)

    def rename_menu(self, menu_item_string: str, new_name: str) -> None:
        """
        Renames a menu item.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~RenameMenu.html
        """
        in_menu_item_string = VARIANT(VT_BSTR, menu_item_string)
        in_new_name = VARIANT(VT_BSTR, new_name)

        self.com_object.RenameMenu(in_menu_item_string, in_new_name)

    def set_status_bar_text(self, message_string: str) -> None:
        """
        Displays a text string in the main status bar area to the left of the status bar.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~SetStatusBarText.html
        """
        in_message_string = VARIANT(VT_BSTR, message_string)
        self.com_object.SetStatusBarText(in_message_string)

    def show_model_window(self, lp_model_window: IModelWindow) -> None:
        """
        Shows a client model window.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame~ShowModelWindow.html
        """
        in_lp_model_window = VARIANT(VT_BYREF | VT_DISPATCH, lp_model_window.com_object)
        self.com_object.ShowModelWindow(in_lp_model_window)



"""
PYSWX PART TOOLS
"""

from pyswx.tools.part_tools.export_part import export_part
from pyswx.tools.part_tools.open_associated_drawing import open_associated_drawing
from pyswx.tools.part_tools.open_part import open_part



"""
swCustomInfoSetResult_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCustomInfoSetResult_e.html

Status: 🟢
"""

from enum import IntEnum


class SWCustomInfoSetResultE(IntEnum):
    """Result codes when setting custom properties."""

    SW_CUSTOM_INFO_SET_RESULT_OK = 0  # Success
    SW_CUSTOM_INFO_SET_RESULT_NOT_PRESENT = 1  # Property does not exist
    SW_CUSTOM_INFO_SET_RESULT_TYPE_MISMATCH = 2  # Incorrect value type
    SW_CUSTOM_INFO_SET_RESULT_LINKED_PROP = (
        3  # Property is linked and cannot be modified
    )



"""
PART TOOLS // OPEN DRAWING
"""

from pathlib import Path
from typing import Tuple

from pyswx.api.sldworks.interfaces.i_document_specification import (
    IDocumentSpecification,
)
from pyswx.api.sldworks.interfaces.i_drawing_doc import IDrawingDoc
from pyswx.api.sldworks.interfaces.i_model_doc_2 import IModelDoc2
from pyswx.api.sldworks.interfaces.i_sldworks import ISldWorks
from pyswx.api.swconst.enumerations import SWDocumentTypesE
from pyswx.api.swconst.enumerations import SWRebuildOnActivationOptionsE
from pyswx.api.swconst.enumerations import SWSaveAsOptionsE
from pyswx.tools.part_tools.open_part import open_part


def open_associated_drawing(
    swx: ISldWorks,
    part_path: Path,
    close_document: bool = False,
    save_document: bool = True,
    document_specification: IDocumentSpecification | None = None,
) -> Tuple[IModelDoc2, IDrawingDoc]:
    """
    Open the drawing associated with a SolidWorks part file.
    Uses the same name in the same folder as the part file with a .SLDDRW extension.

    Args:
        swx (ISldWorks): The SolidWorks application instance.
        part_path (Path): The path to the SolidWorks part file.
        close_document (bool, optional): Whether to close the part document after export. Defaults to False.
        save_document (bool, optional): Whether to save the part document before closing. Defaults to True.
        document_specification (IDocumentSpecification | None, optional): The document specification to use. Defaults to None.

    Returns:
        Tuple[IModelDoc2, IDrawingDoc]: A tuple containing the model document and drawing document.

    Raises:
        ValueError: Raised if no active document is found.
        ValueError: Raised if the active document is not a part.
        FileNotFoundError: Raised if the drawing file does not exist.
        ValueError: Raised if the active document is not a drawing.
    """

    part_model, _ = open_part(
        swx=swx, part_path=part_path, document_specification=document_specification
    )

    drawing_path = part_model.get_path_name().with_suffix(".SLDDRW")
    if not drawing_path.exists():
        raise FileNotFoundError(f"Drawing file does not exist: {drawing_path}")

    drawing_model = swx.open_doc6(
        file_name=drawing_path, file_type=SWDocumentTypesE.SW_DOC_DRAWING
    )
    drawing_model = swx.activate_doc_3(
        name=drawing_path,
        use_user_preferences=False,
        option=SWRebuildOnActivationOptionsE.SW_REBUILD_ACTIVE_DOC,
    )

    model_type = drawing_model.get_type()
    if model_type != SWDocumentTypesE.SW_DOC_DRAWING:
        raise ValueError(f"Active document is not a drawing: {model_type.name}")

    if save_document:
        part_model.save_3(options=SWSaveAsOptionsE.SW_SAVE_AS_OPTIONS_SILENT)

    if close_document:
        swx.quit_doc(name=part_model.get_path_name())

    return drawing_model, IDrawingDoc(drawing_model.com_object)



"""
swMessageBoxBtn_e Enumeration

Defines the control button sets available for SolidWorks message boxes.

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swMessageBoxBtn_e.html

Status: 🟢
"""

from enum import IntEnum


class SWMessageBoxBtnE(IntEnum):
    """Message box control button configurations."""

    SW_MB_ABORT_RETRY_IGNORE = 1  # Buttons: Abort, Retry, Ignore
    SW_MB_OK = 2  # Button: OK
    SW_MB_OK_CANCEL = 3  # Buttons: OK, Cancel
    SW_MB_RETRY_CANCEL = 4  # Buttons: Retry, Cancel
    SW_MB_YES_NO = 5  # Buttons: Yes, No
    SW_MB_YES_NO_CANCEL = 6  # Buttons: Yes, No, Cancel



"""
swFileLoadWarning_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swFileLoadWarning_e.html

Status: 🟢
"""

from enum import IntEnum


class SWFileLoadWarningE(IntEnum):
    SW_FILELOADWARNING_ID_MISMATCH = 1  #: Internal ID mismatch with referencing document
    SW_FILELOADWARNING_SHARING_VIOLATION = 4  #: Document is being used by another user
    SW_FILELOADWARNING_READ_ONLY = 2  #: Document is read-only
    SW_FILELOADWARNING_NEEDS_REGEN = 32  #: Document needs to be rebuilt
    SW_FILELOADWARNING_MODEL_OUT_OF_DATE = 8192  #: Drawing views are out of date with external models
    SW_FILELOADWARNING_SHEET_SCALE_UPDATE = 16  #: Sheet scale applied to sketch entities
    SW_FILELOADWARNING_DRAWING_ANSI_UPDATE = 8  #: Radial dimension arrows displayed outside
    SW_FILELOADWARNING_REVOLVE_DIM_TOLERANCE = 4096  #: Tolerances of revolved feature dimensions not synchronized
    SW_FILELOADWARNING_BASE_PART_NOT_LOADED = 64  #: Document defined in context of another existing document not loaded
    SW_FILELOADWARNING_ALREADY_OPEN = 128  #: Document is already open
    SW_FILELOADWARNING_AUTOMATIC_REPAIR = 262144  #: Non-critical data in the document was automatically repaired
    SW_FILELOADWARNING_CRITICAL_DATA_REPAIR = 524288  #: Critical data in the document was automatically repaired
    SW_FILELOADWARNING_DIMENSIONS_REFERENCED_INCORRECTLY_TO_MODELS = (
        16384  #: Dimensions referenced incorrectly to models
    )
    SW_FILELOADWARNING_DRAWING_SF_SYMBOL_CONVERT = 2048  #: Prompt to convert drawing's surface finish symbols
    SW_FILELOADWARNING_DRAWINGS_ONLY_RAPIDDRAFT = 256  #: Only RapidDraft format conversion can take place
    SW_FILELOADWARNING_INVISIBLE_DOC_LINKED_DESIGN_TABLE_UPDATE_FAIL = (
        65536  #: Invisible document with linked design table update failed
    )
    SW_FILELOADWARNING_MISSING_DESIGN_TABLE = 131072  #: Design table is missing
    SW_FILELOADWARNING_MISSING_EXTERNAL_REFERENCES = 1048576  #: One or more references are missing
    SW_FILELOADWARNING_COMPONENT_MISSING_REFERENCED_CONFIG = 32768  #: Component missing referenced configuration
    SW_FILELOADWARNING_VIEW_MISSING_REFERENCED_CONFIG = 1024  #: Drawing view referencing missing configuration
    SW_FILELOADWARNING_VIEW_ONLY_RESTRICTIONS = 512  #: Document is view-only with non-default configuration set



"""
swMessageBoxResult_e Enumeration

Defines the possible results returned from SolidWorks message boxes.

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swMessageBoxResult_e.html

Status: 🟢
"""

from enum import IntEnum


class SWMessageBoxResultE(IntEnum):
    """Results of user interaction with a message box."""

    SW_MB_HIT_ABORT = 1  # Abort was clicked
    SW_MB_HIT_CANCEL = 7  # Cancel was clicked
    SW_MB_HIT_IGNORE = 2  # Ignore was clicked
    SW_MB_HIT_NO = 3  # No was clicked
    SW_MB_HIT_OK = 4  # OK was clicked
    SW_MB_HIT_RETRY = 5  # Retry was clicked
    SW_MB_HIT_YES = 6  # Yes was clicked



"""
swChildComponentInBOMOption_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swChildComponentInBOMOption_e.html

Status: 🟢
"""

from enum import IntEnum


class SWChildComponentInBOMOptionE(IntEnum):
    """Child component BOM display options for assemblies."""

    SW_CHILD_COMPONENT_HIDE = (
        1  # Child components might be listed individually depending on BOM settings
    )
    SW_CHILD_COMPONENT_SHOW = 2  # Subassembly appears as a single item in the BOM
    SW_CHILD_COMPONENT_PROMOTE = (
        3  # Assembly's configuration dissolves in BOM; children promoted one level
    )



"""
swCloseReopenOption_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCloseReopenOption_e.html

Status: 🟢
"""

from enum import IntEnum


class SWCloseReopenOptionE(IntEnum):
    """File close and reopen options. Bitmask."""

    SW_CLOSE_REOPEN_OPTION_READ_ONLY = 1  # Open the document in read-only mode
    SW_CLOSE_REOPEN_OPTION_DISCARD_CHANGES = 2  # Discard changes before reopening
    SW_CLOSE_REOPEN_OPTION_MATCH_SHEET = 4  # Reopen with the same active sheet
    SW_CLOSE_REOPEN_OPTION_EXIT_DETAILING_MODE = 8  # Reopen model drawings as resolved



"""
swExportDataSheetsToExport_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swExportDataSheetsToExport_e.html

Status: 🟢
"""

from enum import IntEnum


class SWExportDataSheetsToExportE(IntEnum):
    """Export data sheets to export options."""

    SW_EXPORT_DATA_EXPORT_ALL_SHEETS = 1  # Export all sheets
    SW_EXPORT_DATA_EXPORT_CURRENT_SHEET = 2  # Export only the current sheet
    SW_EXPORT_DATA_EXPORT_SPECIFIED_SHEETS = 3  # Export specified sheets



"""
swConfigTreeSortType_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swConfigTreeSortType_e.html

Status: 🟢
"""

from enum import IntEnum


class SWConfigTreeSortTypeE(IntEnum):
    """Order in which configurations are listed in the ConfigurationManager."""

    SW_SORT_TYPE_HISTORY = 0  # Sorted by creation date (earliest to latest)
    SW_SORT_TYPE_NUMERIC = 1  # Sorted by ascending numeric or alphanumeric value
    SW_SORT_TYPE_LITERAL = 2  # Sorted alphabetically
    SW_SORT_TYPE_DESIGN_TABLE = 3  # Sorted by order in the design table



"""
IModelDocExtension Interface Members

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IModelDocExtension_members.html

Status: 🔴
"""

from pathlib import Path
from typing import Tuple

from pythoncom import VT_BSTR
from pythoncom import VT_BYREF
from pythoncom import VT_DISPATCH
from pythoncom import VT_I4
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.sldworks.interfaces.i_advanced_save_as_options import IAdvancedSaveAsOptions
from pyswx.api.sldworks.interfaces.i_custom_property_manager import ICustomPropertyManager
from pyswx.api.sldworks.interfaces.i_export_pdf_data import IExportPdfData
from pyswx.api.swconst.enumerations import SWFileSaveErrorE
from pyswx.api.swconst.enumerations import SWFileSaveWarningE
from pyswx.api.swconst.enumerations import SWSaveAsOptionsE
from pyswx.api.swconst.enumerations import SWSaveAsVersionE
from pyswx.exceptions import DocumentError


class IModelDocExtension(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"IModelDocExtension({self.com_object})"

    def custom_property_manager(self, config_name: str) -> ICustomPropertyManager:
        """
        Gets the custom property information for this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~CustomPropertyManager.html

        Note:
        In the api definition this is a property, but in the python implementation it is a method, because python
        does not support properties with parameters. The parameter is the configuration name.
        """
        in_config_name = VARIANT(VT_BSTR, config_name)
        return ICustomPropertyManager(self.com_object.CustomPropertyManager(in_config_name))

    def save_as_3(
        self,
        name: Path,
        version: SWSaveAsVersionE,
        options: SWSaveAsOptionsE | None,
        export_data: IExportPdfData | None,
        advanced_save_as_options: IAdvancedSaveAsOptions | None,
    ) -> Tuple[bool, SWFileSaveErrorE | None, SWFileSaveWarningE | None]:
        """
        Saves the active document to the specified name with advanced options.

        Args:
            name (Path): Full pathname of the document to save; the file extension indicates any conversion that should be performed (for example, Part1.igs to save in IGES format) (see Remarks)
            version (SWSaveAsVersionE): Format in which to save this document as defined in swSaveAsVersion_e (see Remarks)
            options (SWSaveAsOptionsE): Option indicating how to save the document as defined in swSaveAsOptions_e (see Remarks)
            export_data (IExportPdfData): IExportPdfData object for exporting drawing sheets to PDF (see Remarks)
            advanced_save_as_options (IAdvancedSaveAsOptions): IAdvancedSaveAsOptions (see Remarks)

        Returns:
        True if the save is successful, false if not

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IModelDocExtension~SaveAs3.html

        Raises:
            DocumentError: Raised if there is an error saving the document.
        """
        in_name = VARIANT(VT_BSTR, str(name))
        in_version = VARIANT(VT_I4, version.value)
        in_options = VARIANT(VT_I4, options.value) if options else VARIANT(VT_I4, 0)
        in_export_data = VARIANT(VT_DISPATCH, export_data.com_object) if export_data else VARIANT(VT_DISPATCH, None)
        in_advanced_save_as_options = (
            VARIANT(VT_DISPATCH, advanced_save_as_options.com_object)
            if advanced_save_as_options
            else VARIANT(VT_DISPATCH, None)
        )

        out_errors = VARIANT(VT_BYREF | VT_I4, None)
        out_warnings = VARIANT(VT_BYREF | VT_I4, None)

        return_errors = None
        return_warnings = None

        com_object = self.com_object.SaveAs3(
            in_name,
            in_version,
            in_options,
            in_export_data,
            in_advanced_save_as_options,
            out_errors,
            out_warnings,
        )

        if out_warnings.value != 0:
            try:
                return_warnings = SWFileSaveWarningE(value=out_warnings.value)
            except:
                return_warnings = SWFileSaveWarningE(value=0)
            self.logger.warning(return_warnings.name)

        if out_errors.value != 0:
            try:
                return_errors = SWFileSaveErrorE(value=out_errors.value)
            except:
                return_errors = SWFileSaveErrorE(value=0)
            self.logger.error(return_errors.name)

        return (com_object, return_errors, return_warnings)



"""
ISldWorks

Provides direct and indirect access to all other interfaces exposed in the SOLIDWORKS API.

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks_members.html

Status: 🟠
"""

from pathlib import Path
from typing import Tuple

from pythoncom import VT_BOOL
from pythoncom import VT_BSTR
from pythoncom import VT_BYREF
from pythoncom import VT_DISPATCH
from pythoncom import VT_I4
from pythoncom import VT_R8
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.sldworks.interfaces.i_document_specification import IDocumentSpecification
from pyswx.api.sldworks.interfaces.i_export_pdf_data import IExportPdfData
from pyswx.api.sldworks.interfaces.i_frame import IFrame
from pyswx.api.sldworks.interfaces.i_model_doc_2 import IModelDoc2
from pyswx.api.swconst.enumerations import SWApplicationTypeE
from pyswx.api.swconst.enumerations import SWCloseReopenErrorE
from pyswx.api.swconst.enumerations import SWCloseReopenOptionE
from pyswx.api.swconst.enumerations import SWDocActivateErrorE
from pyswx.api.swconst.enumerations import SWDocumentTypesE
from pyswx.api.swconst.enumerations import SWDwgPaperSizesE
from pyswx.api.swconst.enumerations import SWExportDataFileType_e
from pyswx.api.swconst.enumerations import SWFileLoadErrorE
from pyswx.api.swconst.enumerations import SWFileLoadWarningE
from pyswx.api.swconst.enumerations import SWMessageBoxBtnE
from pyswx.api.swconst.enumerations import SWMessageBoxIconE
from pyswx.api.swconst.enumerations import SWMessageBoxResultE
from pyswx.api.swconst.enumerations import SWOpenDocOptionsE
from pyswx.api.swconst.enumerations import SWRebuildOnActivationOptionsE
from pyswx.api.swconst.enumerations import SWUserPreferenceStringValueE
from pyswx.exceptions import ArgumentError
from pyswx.exceptions import DocumentError


class ISldWorks(BaseInterface):
    """
    ISldWorks Interface Members
    """

    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"ISldWorks({self.com_object})"

    @property
    def active_doc(self) -> IModelDoc2 | None:
        """
        Gets the currently active document.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks~ActiveDoc.html
        """
        com_object = self.com_object.ActiveDoc
        if com_object:
            return IModelDoc2(com_object)
        return None

    @property
    def application_type(self) -> SWApplicationTypeE:
        """
        Gets the type of this SOLIDWORKS application.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks~ApplicationType.html
        """
        com_object = self.com_object.ApplicationType
        return SWApplicationTypeE(value=com_object)

    @property
    def command_in_progress(self):
        raise NotImplementedError

    @command_in_progress.setter
    def command_in_progress(self, value):
        raise NotImplementedError

    @property
    def enable_background_processing(self):
        raise NotImplementedError

    @enable_background_processing.setter
    def enable_background_processing(self, value):
        raise NotImplementedError

    @property
    def enable_file_menu(self):
        raise NotImplementedError

    @enable_file_menu.setter
    def enable_file_menu(self, value):
        raise NotImplementedError

    @property
    def frame_height(self):
        raise NotImplementedError

    @frame_height.setter
    def frame_height(self, value):
        raise NotImplementedError

    @property
    def frame_left(self):
        raise NotImplementedError

    @frame_left.setter
    def frame_left(self, value):
        raise NotImplementedError

    @property
    def frame_state(self):
        raise NotImplementedError

    @frame_state.setter
    def frame_state(self, value):
        raise NotImplementedError

    @property
    def frame_top(self):
        raise NotImplementedError

    @frame_top.setter
    def frame_top(self, value):
        raise NotImplementedError

    @property
    def frame_width(self):
        raise NotImplementedError

    @frame_width.setter
    def frame_width(self, value):
        raise NotImplementedError

    @property
    def i_active_doc2(self):
        raise NotImplementedError

    @i_active_doc2.setter
    def i_active_doc2(self, value):
        raise NotImplementedError

    @property
    def startup_process_completed(self) -> bool:
        """
        Gets whether the SOLIDWORKS startup process, including loading all startup add-ins, has completed.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~startupprocesscompleted.html
        """
        com_object = self.com_object.StartupProcessCompleted
        return bool(com_object)

    @property
    def task_pane_is_pinned(self):
        raise NotImplementedError

    @task_pane_is_pinned.setter
    def task_pane_is_pinned(self, value):
        raise NotImplementedError

    @property
    def user_control(self) -> bool:
        """
        Gets and sets whether the user has control over the application.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~usercontrol.html
        """
        com_object = self.com_object.UserControl
        return bool(com_object)

    @user_control.setter
    def user_control(self, value: bool):
        self.com_object.UserControl = value

    @property
    def user_control_background(self):
        raise NotImplementedError

    @user_control_background.setter
    def user_control_background(self, value):
        raise NotImplementedError

    @property
    def user_type_lib_references(self):
        raise NotImplementedError

    @user_type_lib_references.setter
    def user_type_lib_references(self, value):
        raise NotImplementedError

    @property
    def visible(self) -> bool:
        """
        Gets whether the SOLIDWORKS application is visible.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks~Visible.html
        """
        return self.com_object.Visible

    @visible.setter
    def visible(self, value: bool):
        self.com_object.Visible = VARIANT(VT_BOOL, value)

    def activate_doc_3(
        self,
        name: Path,
        use_user_preferences: bool,
        option: SWRebuildOnActivationOptionsE,
    ) -> IModelDoc2:
        """
        Activates a loaded document and rebuilds it as specified.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~activatedoc3.html

        Raises:
            DocumentError: Raised if there is an error activating the document.
        """

        in_name = VARIANT(VT_BSTR, name.name)
        in_use_user_preferences = VARIANT(VT_BOOL, use_user_preferences)
        in_option = VARIANT(VT_I4, option)

        out_errors = VARIANT(VT_BYREF | VT_I4, None)

        model_doc = self.com_object.ActivateDoc3(
            in_name,
            in_use_user_preferences,
            in_option,
            out_errors,
        )

        if out_errors.value != 0:
            out_errors = SWDocActivateErrorE(value=out_errors.value)
            raise DocumentError(str(out_errors))

        return IModelDoc2(model_doc)

    def activate_task_pane(self):
        """
        Activates the specified task pane.
        """
        raise NotImplementedError

    def add_callback(self):
        """
        Registers a general purpose callback handler.
        """
        raise NotImplementedError

    def add_file_open_item3(self):
        """
        Adds file types to the File > Open dialog box.
        """
        raise NotImplementedError

    def add_file_save_as_item2(self):
        """
        Adds a file type to the SOLIDWORKS File > Save As dialog box.
        """
        raise NotImplementedError

    def add_item_to_third_party_popup_menu(self):
        """
        Adds menu items to a pop-up (shortcut) menu in a C++ SOLIDWORKS add-in.
        """
        raise NotImplementedError

    def add_item_to_third_party_popup_menu2(self):
        """
        Adds menu items to a pop-up (shortcut) menu in a SOLIDWORKS add-in.
        """
        raise NotImplementedError

    def add_menu(self):
        """
        Adds a menu item to a SOLIDWORKS menu for DLL applications.
        """
        raise NotImplementedError

    def add_menu_item5(self):
        """
        Adds a menu item and image to the SOLIDWORKS user interface.
        """
        raise NotImplementedError

    def add_menu_popup_item3(self):
        """
        Adds a menu item and zero or more submenus to shortcut menus of entities
        of the specified type in documents of the specified type.
        """
        raise NotImplementedError

    def add_menu_popup_item4(self):
        """
        Adds a menu item and zero or more submenus to shortcut menus of features
        of the specified type in documents of the specified type.
        """
        raise NotImplementedError

    def add_toolbar5(self):
        """
        Creates a Windows-style dockable toolbar.
        """
        raise NotImplementedError

    def add_toolbar_command2(self):
        """
        Specifies the application functions to call when a toolbar button is
        clicked or sets a separator.
        """
        raise NotImplementedError

    def allow_failed_feature_creation(self):
        """
        Sets whether to allow the creation of a feature that has rebuild errors.
        """
        raise NotImplementedError

    def arrange_icons(self):
        """
        Arranges the icons in SOLIDWORKS.
        """
        raise NotImplementedError

    def arrange_windows(self):
        """
        Arranges the open windows in SOLIDWORKS.
        """
        raise NotImplementedError

    def block_skinning(self):
        """
        Blocks skinning a window, which prevents a window from looking like a
        SOLIDWORKS window.
        """
        raise NotImplementedError

    def callback(self):
        """
        Allows an out-of-process executable or a SOLIDWORKS macro to call back a function in a SOLIDWORKS add-in DLL.
        """
        raise NotImplementedError

    def checkpoint_converted_document(self):
        """
        Saves the specified open document if its version is older than the version of the SOLIDWORKS product being used.
        """
        raise NotImplementedError

    def close_all_documents(self, include_unsaved: bool) -> bool:
        """
        Closes all open documents in the SOLIDWORKS session.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~closealldocuments.html
        """
        in_include_unsaved = VARIANT(VT_BOOL, include_unsaved)

        com_object = self.com_object.CloseAllDocuments(in_include_unsaved)
        return bool(com_object)

    def close_and_reopen(self, doc: IModelDoc2, option: SWCloseReopenOptionE) -> Tuple[SWCloseReopenErrorE, IModelDoc2]:
        """
        Closes and reopens the specified drawing document without unloading its references from memory.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks~CloseAndReopen.html
        """
        if doc.get_type() != SWDocumentTypesE.SW_DOC_DRAWING:
            raise DocumentError("Document is not a drawing")

        in_doc = VARIANT(VT_DISPATCH, doc.com_object)
        in_option = VARIANT(VT_I4, option.value)

        out_new_doc = VARIANT(VT_BYREF | VT_DISPATCH, None)

        com_object = self.com_object.CloseAndReopen(in_doc, in_option, out_new_doc)
        return (SWCloseReopenErrorE(com_object), IModelDoc2(out_new_doc.value))

    def close_and_reopen2(
        self, doc: IModelDoc2, option: SWCloseReopenOptionE
    ) -> Tuple[SWCloseReopenErrorE, IModelDoc2]:
        """
        Closes and reopens the specified drawing document without unloading its references from memory.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks~CloseAndReopen2.html
        """
        if doc.get_type() != SWDocumentTypesE.SW_DOC_DRAWING:
            raise DocumentError("Document is not a drawing")

        in_doc = VARIANT(VT_DISPATCH, doc.com_object)
        in_option = VARIANT(VT_I4, option.value)

        out_new_doc = VARIANT(VT_BYREF | VT_DISPATCH, None)

        com_object = self.com_object.CloseAndReopen2(in_doc, in_option, out_new_doc)
        return (SWCloseReopenErrorE(com_object), IModelDoc2(out_new_doc.value))

    def close_doc(self, name: str) -> None:
        """
        Closes the specified document.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks~CloseDoc.html
        """
        in_name = VARIANT(VT_BSTR, name)

        self.com_object.CloseDoc(in_name)

    def close_user_notification(self):
        """
        Closes the specified user notification.
        """
        raise NotImplementedError

    def command(self):
        """
        Opens the specified dialog or file.
        """
        raise NotImplementedError

    def copy_appearance(self):
        """
        Copies the appearance of the specified entity to the clipboard.
        """
        raise NotImplementedError

    def copy_document(self):
        """
        Copies a document and optionally updates references to it.
        """
        raise NotImplementedError

    def create_new_window(self):
        """
        Creates a client window containing the active document.
        """
        raise NotImplementedError

    def create_property_manager_page(self):
        """
        Creates a PropertyManager page.
        """
        raise NotImplementedError

    def create_taskpane_view3(self):
        """
        Creates an application-level Task Pane view.
        """
        raise NotImplementedError

    def define_attribute(self):
        """
        Creates an attribute definition, which is the first step in generating attributes.
        """
        raise NotImplementedError

    def define_message_bar(self):
        """
        Called by a SOLIDWORKS add-in, creates a message bar definition object.
        """
        raise NotImplementedError

    def define_user_notification(self):
        """
        Called by a SOLIDWORKS add-in, creates a user notification definition object.
        """
        raise NotImplementedError

    def display_status_bar(self):
        """
        Sets whether to display the status bar.
        """
        raise NotImplementedError

    def document_visible(self):
        """
        Allows the application to control the display of a document in a window upon creation or retrieval.
        """
        raise NotImplementedError

    def download_from_my_solidworks_settings(self):
        """
        Downloads the specified SOLIDWORKS Connected settings to SOLIDWORKS Desktop.
        """
        raise NotImplementedError

    def drag_toolbar_button(self):
        """
        Copies the specified toolbar button from the specified native SOLIDWORKS toolbar or ICommandGroup
        toolbar to the specified native SOLIDWORKS toolbar or ICommandGroup toolbar.
        """
        raise NotImplementedError

    def drag_toolbar_button_from_command_id(self):
        """
        Copies a button to a toolbar using a command ID.
        """
        raise NotImplementedError

    def enum_documents2(self):
        """
        Gets a list of documents currently open in the application.
        """
        raise NotImplementedError

    def exit_app(self) -> None:
        """
        Shuts down SOLIDWORKS.

        Remarks:
        This method is not normally used with macros (*.swp) because it shuts down your SOLIDWORKS session.
        For C++ and Visual Basic projects, ending your program does not guarantee that the SOLIDWORKS processes are closed. During development, you can determine whether processes are left running by checking the Process Viewer and looking for any SLDWORKS processes. Typically, you do not want the SLDWORKS process running after your program has ended. Calling this method ensures that no SOLIDWORKS processes are left running when your program ends.
        However, the CreateObject ("SldWorks.Application") statement used to start up the SOLIDWORKS software either creates a new SOLIDWORKS session or it attaches to an existing SOLIDWORKS session. If the end-user currently has an open SOLIDWORKS session, then your program attaches to it. Performing this method ends that session.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks~ExitApp.html
        """
        self.com_object.ExitApp()

    def export_hole_wizard_item(self):
        """
        Exports data for the specified Hole Wizard standard.
        """
        raise NotImplementedError

    def export_toolbox_item(self):
        """
        Exports data for the specified Toolbox standard.
        """
        raise NotImplementedError

    def frame(self) -> IFrame:
        """
        Gets the SOLIDWORKS main frame.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IFrame_members.html
        """
        return IFrame(self.com_object.Frame)

    def get_3dexperience_state(self):
        """
        Gets the current state of SOLIDWORKS Connected.
        """
        raise NotImplementedError

    def get_active_configuration_name(self, file_path_name: Path) -> str:
        """
        Gets the name of the active configuration in the specified SOLIDWORKS document.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~getactiveconfigurationname.html
        """
        if file_path_name.exists():
            in_file_path_name = VARIANT(VT_BSTR, str(file_path_name))
            return str(self.com_object.GetActiveConfigurationName(in_file_path_name))
        else:
            raise FileNotFoundError(file_path_name)

    def get_active_display_pane(self):
        """
        Gets the active Display Pane.
        """
        raise NotImplementedError

    def get_add_in_object(self):
        """
        Gets an add-in object for the specified SOLIDWORKS add-in.
        """
        raise NotImplementedError

    def get_apply_selection_filter(self):
        """
        Gets the current state of the selection filter.
        """
        raise NotImplementedError

    def get_batch_uploaded_files_info(self):
        """
        Gets the files uploaded to 3DEXPERIENCE during a batch process.
        """
        raise NotImplementedError

    def get_build_numbers2(self):
        """
        Gets the build, major revision, and hot fix numbers of the SOLIDWORKS application.
        """
        raise NotImplementedError

    def get_button_position(self):
        """
        Gets the center coordinates of the specified SOLIDWORKS toolbar button.
        """
        raise NotImplementedError

    def get_collision_detection_manager(self):
        """
        Gets the collision detection manager.
        """
        raise NotImplementedError

    def get_color_table(self):
        """
        Gets a color table from the SOLIDWORKS application.
        """
        raise NotImplementedError

    def get_command_id(self):
        """
        Gets the SOLIDWORKS command ID for an instance of an add-in's control (e.g., toolbar button).
        """
        raise NotImplementedError

    def get_command_manager(self):
        """
        Gets the CommandManager for the specified add-in.
        """
        raise NotImplementedError

    def get_configuration_count(self):
        """
        Gets the number of configurations in the SOLIDWORKS document, whether the document is opened or closed.
        """
        raise NotImplementedError

    def get_configuration_names(self):
        """
        Gets the names of the configurations in this SOLIDWORKS document, whether opened or closed.
        """
        raise NotImplementedError

    def get_current_file_user(self):
        """
        Gets the name of the user who has the specified document open.
        """
        raise NotImplementedError

    def get_current_language(self):
        """
        Gets the current language used by SOLIDWORKS.
        """
        raise NotImplementedError

    def get_current_license_type(self):
        """
        Gets the type of license for the current SOLIDWORKS session.
        """
        raise NotImplementedError

    def get_current_macro_path_folder(self):
        """
        Gets the name of the folder where the macro resides.
        """
        raise NotImplementedError

    def get_current_macro_path_name(self):
        """
        Gets the path name for the macro currently running.
        """
        raise NotImplementedError

    def get_current_working_directory(self):
        """
        Gets the current working directory being used by the SOLIDWORKS application.
        """
        raise NotImplementedError

    def get_data_folder(self):
        """
        Gets the data directory name currently used by SOLIDWORKS.
        """
        raise NotImplementedError

    def get_document_count(self) -> int:
        """
        Gets the number of open documents in the current SOLIDWORKS session.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~getdocumentcount.html
        """
        com_object = self.com_object.GetDocumentCount
        return int(com_object)

    def get_document_dependencies2(self):
        """
        Gets all of the model dependencies for a document.
        """
        raise NotImplementedError

    def get_documents(self):
        """
        Gets the open documents in this SOLIDWORKS session.
        """
        raise NotImplementedError

    def get_document_template(self):
        """
        Gets the name of the document template usable in NewDocument or INewDocument2.
        """
        raise NotImplementedError

    def get_document_visible(self):
        """
        Gets the visibility of the document to open.
        """
        raise NotImplementedError

    def get_environment(self):
        """
        Gets the IEnvironment object.
        """
        raise NotImplementedError

    def get_error_messages(self):
        """
        Gets the last 20 messages issued by SOLIDWORKS in the current session.
        """
        raise NotImplementedError

    def get_executable_path(self):
        """
        Gets the path to the SOLIDWORKS executable, sldworks.exe.
        """
        raise NotImplementedError

    def get_export_file_data(self, file_type: SWExportDataFileType_e) -> IExportPdfData:
        """
        Gets the data interface for the specified file type to which to export one or more drawing sheets.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SOLIDWORKS.Interop.sldworks~SOLIDWORKS.Interop.sldworks.ISldWorks~GetExportFileData.html
        """
        in_file_type = VARIANT(VT_I4, file_type.value)

        com_object = self.com_object.GetExportFileData(in_file_type)

        return IExportPdfData(com_object)

    def get_file_plmid(self):
        """
        Gets the Product Lifecycle Management (PLM) ID of the specified file stored in 3DEXPERIENCE.
        """
        raise NotImplementedError

    def get_first_document(self):
        """
        Gets the document that was opened first in this SOLIDWORKS session.
        """
        raise NotImplementedError

    def get_gtol_format_data(self):
        """
        Gets the Gtol format and XML schema versions supported by this version of SOLIDWORKS.
        """
        raise NotImplementedError

    def get_gtol_frame_xml_schema(self):
        """
        Gets the XML schema for Gtol frame symbol XML.
        """
        raise NotImplementedError

    def get_hole_standards_data(self):
        """
        Gets the hole standards for the specified hole type.
        """
        raise NotImplementedError

    def get_image_size(self):
        """
        Gets small, medium, and large image sizes for the current DPI setting of the display device.
        Also returns the default image size for images not based on the SOLIDWORKS icon size setting.
        """
        raise NotImplementedError

    def get_import_file_data(self):
        """
        Gets the IGES or DXF/DWG import data for the specified file.
        """
        raise NotImplementedError

    def get_interface_brightness_theme_colors(self):
        """
        Gets the theme and background colors of the SOLIDWORKS interface.
        """
        raise NotImplementedError

    def get_last_save_error(self):
        """
        Gets the last save error issued by Microsoft in the current session.
        """
        raise NotImplementedError

    def get_last_toolbar_id(self):
        """
        Gets the ID of the last toolbar added to the CommandManager.
        """
        raise NotImplementedError

    def get_latest_supported_file_version(self):
        """
        Gets the version number that this instance of SOLIDWORKS can read and write.
        """
        raise NotImplementedError

    def get_line_styles(self):
        """
        Gets all of the line styles in the specified file.
        """
        raise NotImplementedError

    def get_localized_menu_name(self):
        """
        Gets a localized menu name for the specified menu ID.
        """
        raise NotImplementedError

    def get_macro_methods(self):
        """
        Gets the names of the modules in the specified macro.
        """
        raise NotImplementedError

    def get_mass_properties2(self):
        """
        Gets the mass properties from the specified document and configuration.
        """
        raise NotImplementedError

    def get_material_database_count(self):
        """
        Gets the number of material databases.
        """
        raise NotImplementedError

    def get_material_databases(self):
        """
        Gets the names of the material databases.
        """
        raise NotImplementedError

    def get_material_schema_path_name(self):
        """
        Gets the path of the XML material schema file.
        """
        raise NotImplementedError

    def get_math_utility(self):
        """
        Gets the IMathUtility interface.
        """
        raise NotImplementedError

    def get_menu_strings(self):
        """
        Gets the name of the parent menu for the specified menu command.
        """
        raise NotImplementedError

    def get_modeler(self):
        """
        Gets the IModeler interface.
        """
        raise NotImplementedError

    def get_mouse_drag_mode(self):
        """
        Gets whether the specified command-mouse mode is in effect.
        """
        raise NotImplementedError

    def get_open_doc_spec(self, file_name: Path) -> IDocumentSpecification:
        """
        Gets the specifications to use when opening a model document.

        Args:
            file_name (Path): Path and file name of the document to open

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~getopendocspec.html
        """
        com_object = self.com_object.GetOpenDocSpec(str(file_name))
        return IDocumentSpecification(com_object)

    def get_open_document(self, doc_name: str) -> IModelDoc2 | None:
        """
        Gets the open document with the specified name.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~getopendocument.html
        """
        in_doc_name = VARIANT(VT_BSTR, doc_name)
        com_object = self.com_object.GetOpenDocument(in_doc_name)
        if com_object:
            return IModelDoc2(com_object)

    def get_open_document_by_name(self, document_name: str) -> IModelDoc2 | None:
        """
        Gets the open document by the specified name.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~getopendocumentbyname.html
        """
        in_document_name = VARIANT(VT_BSTR, document_name)
        com_object = self.com_object.GetOpenDocumentByName(in_document_name)
        if com_object:
            return IModelDoc2(com_object)

    def get_opened_file_info(self):
        """
        Gets the name and options of the last model successfully opened by SOLIDWORKS.
        """
        raise NotImplementedError

    def get_open_file_name2(self):
        """
        Prompts the user for the name of the file to open.
        """
        raise NotImplementedError

    def get_preview_bitmap(self):
        """
        Gets the preview bitmap for the specified configuration of a document.
        """
        raise NotImplementedError

    def get_preview_bitmap_file(self):
        """
        Gets the preview bitmap of a document and saves it as a Windows bitmap file.
        """
        raise NotImplementedError

    def get_process_id(self):
        """
        Gets the process ID of the current SOLIDWORKS session.
        """
        raise NotImplementedError

    def get_ray_trace_renderer(self):
        """
        Gets a ray-trace rendering engine.
        """
        raise NotImplementedError

    def get_recent_files(self):
        """
        Gets a list of the most recently used files.
        """
        raise NotImplementedError

    def get_routing_settings(self):
        """
        Gets the routing settings.
        """
        raise NotImplementedError

    def get_running_command_info(self):
        """
        Gets command or PropertyManager page ID, title, and UI activation state.
        """
        raise NotImplementedError

    def get_safe_array_utility(self):
        """
        Gets the ISafeArrayUtility object.
        """
        raise NotImplementedError

    def get_save_to_3dexperience_options(self):
        """
        Initializes save options for a SOLIDWORKS Connected document.
        """
        raise NotImplementedError

    def get_search_folders(self):
        """
        Gets the current folder search path for referenced documents.
        """
        raise NotImplementedError

    def get_selection_filter(self):
        """
        Gets the current selection filter settings for the specified item type.
        """
        raise NotImplementedError

    def get_selection_filters(self):
        """
        Gets all active selection filters.
        """
        raise NotImplementedError

    def get_sso_formatted_url(self):
        """
        Formats the specified URL for single sign-on (SSO).
        """
        raise NotImplementedError

    def get_template_sizes(self):
        """
        Gets the sheet properties from a template document.
        """
        raise NotImplementedError

    def get_toolbar_dock2(self):
        """
        Gets the docking state of the toolbar.
        """
        raise NotImplementedError

    def get_toolbar_state2(self):
        """
        Gets the state of the toolbar.
        """
        raise NotImplementedError

    def get_toolbar_visibility(self):
        """
        Gets whether the specified toolbar is visible.
        """
        raise NotImplementedError

    def get_user_preference_double_value(self):
        """
        Gets system default user preference values of type double.
        """
        raise NotImplementedError

    def get_user_preference_integer_value(self):
        """
        Gets system default user preference values of type integer.
        """
        raise NotImplementedError

    def get_user_preference_string_list_value(self):
        """
        Gets the name of the DXF mapping file or string list user preferences.
        """
        raise NotImplementedError

    def get_user_preference_string_value(self, user_preference: SWUserPreferenceStringValueE) -> str:
        """
        Gets system default user preference values of type string.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~getuserpreferencestringvalue.html
        """
        in_user_preference = VARIANT(VT_I4, user_preference.value)

        com_object = self.com_object.GetUserPreferenceStringValue(in_user_preference)
        return str(com_object)

    def get_user_preference_toggle(self):
        """
        Gets document default user preference toggle values.
        """
        raise NotImplementedError

    def get_user_progress_bar(self):
        """
        Gets a user progress indicator object.
        """
        raise NotImplementedError

    def get_user_type_lib_reference_count(self):
        """
        Gets the number of user-specified type library references.
        """
        raise NotImplementedError

    def get_user_unit(self):
        """
        Gets an empty IUserUnit object of the specified type.
        """
        raise NotImplementedError

    def hide_bubble_tooltip(self):
        """
        Hides the bubble tooltip displayed by ISldWorks::ShowBubbleTooltipAt2.
        """
        raise NotImplementedError

    def hide_toolbar2(self):
        """
        Hides a toolbar created with ISldWorks::AddToolbar5.
        """
        raise NotImplementedError

    def i_activate_doc3(self):
        """
        Activates a document that has already been loaded.
        This document becomes the active document and a pointer to it is returned.
        """
        raise NotImplementedError

    def i_copy_document(self):
        """
        Copies a document and optionally updates references to it.
        """
        raise NotImplementedError

    def i_create_property_manager_page(self):
        """
        Creates a PropertyManager page.
        """
        raise NotImplementedError

    def i_define_attribute(self):
        """
        Creates an attribute definition, which is the first step in generating attributes.
        """
        raise NotImplementedError

    def i_get_color_table(self):
        """
        Gets a color table from the SOLIDWORKS application.
        """
        raise NotImplementedError

    def i_get_configuration_names(self):
        """
        Gets the names of the configurations in this SOLIDWORKS document, opened or closed.
        """
        raise NotImplementedError

    def i_get_document_dependencies2(self):
        """
        Gets all of the model dependencies for a document.
        """
        raise NotImplementedError

    def i_get_document_dependencies_count2(self):
        """
        Gets the size of the array needed for a call to i_get_document_dependencies2.
        """
        raise NotImplementedError

    def i_get_documents(self):
        """
        Gets the open documents in this SOLIDWORKS session.
        """
        raise NotImplementedError

    def i_get_environment(self):
        """
        Gets the IEnvironment object.
        """
        raise NotImplementedError

    def i_get_first_document2(self):
        """
        Gets the document opened first in this SOLIDWORKS session.
        """
        raise NotImplementedError

    def i_get_mass_properties2(self):
        """
        Gets the mass properties from the specified document and configuration.
        """
        raise NotImplementedError

    def i_get_material_databases(self):
        """
        Gets the names of the material databases.
        """
        raise NotImplementedError

    def i_get_math_utility(self):
        """
        Gets the IMathUtility interface.
        """
        raise NotImplementedError

    def i_get_modeler(self):
        """
        Gets the IModeler interface.
        """
        raise NotImplementedError

    def i_get_open_document_by_name2(self):
        """
        Gets the open document with the specified name.
        """
        raise NotImplementedError

    def i_get_ray_trace_renderer(self):
        """
        Gets a ray-trace rendering engine.
        """
        raise NotImplementedError

    def i_get_selection_filters(self):
        """
        Gets all active selection filters.
        """
        raise NotImplementedError

    def i_get_selection_filters_count(self):
        """
        Gets the number of active selection filters.
        """
        raise NotImplementedError

    def i_get_template_sizes(self):
        """
        Gets the sheet properties from a template document.
        """
        raise NotImplementedError

    def i_get_user_type_lib_references(self):
        """
        Gets the specified user-specified type library references.
        """
        raise NotImplementedError

    def i_get_user_unit(self):
        """
        Gets an empty IUserUnit object of the specified type.
        """
        raise NotImplementedError

    def i_get_version_history_count(self):
        """
        Gets the size of the array required to hold data returned by IVersionHistory.
        """
        raise NotImplementedError

    def i_move_document(self):
        """
        Moves a document and optionally updates references to it.
        """
        raise NotImplementedError

    def import_hole_wizard_item(self):
        """
        Imports data for the specified Hole Wizard standard.
        """
        raise NotImplementedError

    def import_toolbox_item(self):
        """
        Imports data for the specified Toolbox standard.
        """
        raise NotImplementedError

    def i_new_document2(self):
        """
        Creates a new document based on the specified template.
        """
        raise NotImplementedError

    def install_quick_tip_guide(self):
        """
        Implements your add-in's copy of the Quick Tips.
        """
        raise NotImplementedError

    def i_remove_user_type_lib_references(self):
        """
        Removes the user-specified type library references.
        """
        raise NotImplementedError

    def is_background_processing_completed(self):
        """
        Gets whether SOLIDWORKS has finished background processing a drawing document.
        """
        raise NotImplementedError

    def is_command_enabled(self):
        """
        Gets whether the specified command is enabled.
        """
        raise NotImplementedError

    def i_set_selection_filters(self):
        """
        Sets the status for multiple selection filters.
        """
        raise NotImplementedError

    def i_set_user_type_lib_references(self):
        """
        Sets the user-specified type library references.
        """
        raise NotImplementedError

    def is_rapid_draft(self):
        """
        Gets whether the specified drawing file is in SOLIDWORKS Detached format.
        """
        raise NotImplementedError

    def is_same(self):
        """
        Gets whether the two specified objects are the same object.
        """
        raise NotImplementedError

    def is_task_pane_expanded(self):
        """
        Gets whether the Task Pane is expanded.
        """
        raise NotImplementedError

    def is_task_pane_visible(self):
        """
        Gets whether the Task Pane is visible.
        """
        raise NotImplementedError

    def i_version_history(self):
        """
        Gets a list of strings indicating the versions in which a model was saved.
        """
        raise NotImplementedError

    def load_add_in(self):
        """
        Loads the specified add-in in SOLIDWORKS.
        """
        raise NotImplementedError

    def load_admin_settings_file(self):
        """
        Loads the specified *.sldsettings file into SOLIDWORKS Connected.
        """
        raise NotImplementedError

    def load_file4(self):
        """
        Loads a third-party native CAD file into a new SOLIDWORKS document using 3D Interconnect.
        """
        raise NotImplementedError

    def move_document(self):
        """
        Moves a document and optionally updates references to it.
        """
        raise NotImplementedError

    def new_document(
        self, template_name: Path, paper_size: SWDwgPaperSizesE, width: float | None = None, height: float | None = None
    ) -> IModelDoc2 | None:
        """
        Creates a new document based on the specified template.

        Reference:
        https://help.solidworks.com/2022/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~newdocument.html
        """
        if paper_size == SWDwgPaperSizesE.SW_DWG_PAPERS_USER_DEFINED and not width and not height:
            raise ArgumentError(
                "Argument width and height must not be None when paper_size is 'SW_DWG_PAPERS_USER_DEFINED'"
            )

        in_template_name = VARIANT(VT_BSTR, str(template_name))
        in_paper_size = VARIANT(VT_I4, paper_size.value)
        in_width = VARIANT(VT_R8, width or 0)
        in_height = VARIANT(VT_R8, height or 0)

        com_object = self.com_object.NewDocument(in_template_name, in_paper_size, in_width, in_height)
        return IModelDoc2(com_object) if com_object else None

    def open_doc6(
        self,
        file_name: Path,
        file_type: SWDocumentTypesE,
        options: SWOpenDocOptionsE | None = None,
        configuration: str | None = None,
    ) -> IModelDoc2:
        """
        Opens an existing document and returns a pointer to the document object.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~opendoc6.html
        """

        if not file_name.exists():
            raise FileNotFoundError(file_name)

        in_file_name = VARIANT(VT_BSTR, str(file_name))
        in_file_type = VARIANT(VT_I4, file_type.value)
        in_options = VARIANT(VT_I4, options.value) if options else 0
        in_configuration = VARIANT(VT_BSTR, configuration) if configuration else ""

        out_errors = VARIANT(VT_BYREF | VT_I4, None)
        out_warnings = VARIANT(VT_BYREF | VT_I4, None)

        com_object = self.com_object.OpenDoc6(
            in_file_name,
            in_file_type,
            in_options,
            in_configuration,
            out_errors,
            out_warnings,
        )

        if out_warnings.value != 0:
            out_warnings = SWFileLoadWarningE(value=out_warnings.value)
            self.logger.warning(out_warnings.name)

        if out_errors.value != 0:
            out_errors = SWFileLoadErrorE(value=out_errors.value)
            raise DocumentError(out_errors.name)

        return IModelDoc2(com_object)

    def open_doc7(self, specification: IDocumentSpecification) -> IModelDoc2 | None:
        """
        Opens an existing document using a specification object and returns a pointer to the document object.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.isldworks~opendoc7.html
        """
        if not specification.file_name.exists():
            raise FileNotFoundError(specification.file_name)

        in_specification = VARIANT(VT_DISPATCH, specification.com_object)
        com_object = self.com_object.OpenDoc7(in_specification)

        if in_specification.value.Warning != 0:
            try:
                out_warnings = SWFileLoadWarningE(value=in_specification.value.Warning)
                self.logger.warning(out_warnings.name)
            except ValueError:
                self.logger.error(f"Unknown SWFileLoadWarning: {in_specification.value.Warning}")

        if in_specification.value.Error != 0:
            out_errors = SWFileLoadErrorE(value=in_specification.value.Error)
            raise DocumentError(str(out_errors))

        return IModelDoc2(com_object) if com_object else None

    def paste_appearance(self):
        """
        Applies an appearance copied to the clipboard to the specified entity.
        """
        raise NotImplementedError

    def post_message_to_application(self):
        """
        Posts a message to the application that invoked this method.
        """
        raise NotImplementedError

    def post_message_to_application_x64(self):
        """
        Posts a message to the application in 64-bit environments.
        """
        raise NotImplementedError

    def pre_select_dwg_template_size(self):
        """
        Sets which drawing template to use when creating a drawing.
        """
        raise NotImplementedError

    def preset_new_drawing_parameters(self):
        """
        Presets drawing template and sheet size parameters to avoid showing the sheet format dialog.
        """
        raise NotImplementedError

    def preview_doc(self):
        """
        Displays a preview of a document to a specified window.
        """
        raise NotImplementedError

    def preview_doc_x64(self):
        """
        Displays a preview of a document in 64-bit applications.
        """
        raise NotImplementedError

    def quit_doc(self, name: Path) -> None:
        """
        Closes the specified document without saving changes.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ISldWorks~QuitDoc.html
        """
        self.com_object.QuitDoc(str(name))

    def record_line(self):
        """
        Adds a line of code to a VBA macro and the SOLIDWORKS journal file.
        """
        raise NotImplementedError

    def record_line_csharp(self):
        """
        Adds a line of code to a C# macro and the SOLIDWORKS journal file.
        """
        raise NotImplementedError

    def record_line_vbnet(self):
        """
        Adds a line of code to a VB.NET macro and the SOLIDWORKS journal file.
        """
        raise NotImplementedError

    def refresh_quick_tip_window(self):
        """
        Notifies SOLIDWORKS that the add-in's state changed to refresh quick tip content.
        """
        raise NotImplementedError

    def refresh_taskpane_content(self):
        """
        Refreshes the Design Library tab in the Task Pane.
        """
        raise NotImplementedError

    def register_third_party_popup_menu(self):
        """
        Registers a third-party shortcut menu.
        """
        raise NotImplementedError

    def register_tracking_definition(self):
        """
        Registers a tracking definition.
        """
        raise NotImplementedError

    def remove_callback(self):
        """
        Unregisters a general-purpose callback handler.
        """
        raise NotImplementedError

    def remove_file_open_item2(self):
        """
        Removes a file type from the File > Open dialog box added previously.
        """
        raise NotImplementedError

    def remove_file_save_as_item2(self):
        """
        Removes a file type from the File > Save As dialog box added previously.
        """
        raise NotImplementedError

    def remove_from_menu(self):
        """
        Removes the specified command from menus and toolbars.
        """
        raise NotImplementedError

    def remove_from_popup_menu(self):
        """
        Removes a menu item from specified context-sensitive (popup) menus.
        """
        raise NotImplementedError

    def remove_item_from_third_party_popup_menu(self):
        """
        Removes a menu item and icon from a third-party shortcut menu.
        """
        raise NotImplementedError

    def remove_menu(self):
        """
        Removes a menu item from the specified document frame.
        """
        raise NotImplementedError

    def remove_menu_popup_item2(self):
        """
        Removes an item on a pop-up (shortcut) menu.
        """
        raise NotImplementedError

    def remove_toolbar2(self):
        """
        Removes a toolbar created with AddToolbar5.
        """
        raise NotImplementedError

    def remove_user_type_lib_references(self):
        """
        Removes the user-specified type library references.
        """
        raise NotImplementedError

    def replace_referenced_document(self):
        """
        Replaces a referenced document.
        """
        raise NotImplementedError

    def reset_preset_drawing_parameters(self):
        """
        Resets drawing template behavior to default after preset parameters were set.
        """
        raise NotImplementedError

    def reset_untitled_count(self):
        """
        Resets the index of untitled documents.
        """
        raise NotImplementedError

    def restore_settings(self):
        """
        Restores SOLIDWORKS settings from a specified *.sldreg file.
        """
        raise NotImplementedError

    def resume_skinning(self):
        """
        Resumes skinning windows.
        """
        raise NotImplementedError

    def revision_number(self):
        """
        Gets the version number of the SOLIDWORKS installation.
        """
        raise NotImplementedError

    def run_attached_macro(self):
        """
        Runs the specified attached macro, module, and procedure.
        """
        raise NotImplementedError

    def run_batch_save_process(self):
        """
        Batch saves files to 3DEXPERIENCE.
        """
        raise NotImplementedError

    def run_command(self):
        """
        Runs the specified SOLIDWORKS command.
        """
        raise NotImplementedError

    def run_macro2(self):
        """
        Runs a macro from a project file.
        """
        raise NotImplementedError

    def save_settings(self):
        """
        Saves SOLIDWORKS settings to a specified *.sldreg file.
        """
        raise NotImplementedError

    def send_msg_to_user2(self, message: str, icon: SWMessageBoxIconE, buttons: SWMessageBoxBtnE):
        """
        Displays a message box that requires user interaction before continuing.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SOLIDWORKS.Interop.sldworks~SOLIDWORKS.Interop.sldworks.ISldWorks~SendMsgToUser2.html
        """
        in_message = VARIANT(VT_BSTR, message)
        in_icon = VARIANT(VT_I4, icon.value)
        in_button = VARIANT(VT_I4, buttons.value)

        com_object = self.com_object.SendMsgToUser2(in_message, in_icon, in_button)
        return SWMessageBoxResultE(com_object)

    def set_addin_callback_info2(self):
        """
        Sets add-in callback commands.
        """
        raise NotImplementedError

    def set_apply_selection_filter(self):
        """
        Sets the current state of the selection filter.
        """
        raise NotImplementedError

    def set_current_working_directory(self):
        """
        Sets the current working directory used by SOLIDWORKS.
        """
        raise NotImplementedError

    def set_missing_reference_path_name(self):
        """
        Sets the missing reference file name for ReferenceNotFoundNotify event.
        """
        raise NotImplementedError

    def set_mouse_drag_mode(self):
        """
        Sets the command-mouse mode.
        """
        raise NotImplementedError

    def set_multiple_filenames_prompt(self):
        """
        Sets new filenames to open in response to a prompt event.
        """
        raise NotImplementedError

    def set_new_filename(self):
        """
        Sets the name of the new SOLIDWORKS file.
        """
        raise NotImplementedError

    def set_prompt_filename2(self):
        """
        Sets the file to open in response to a SOLIDWORKS event.
        """
        raise NotImplementedError

    def set_search_folders(self):
        """
        Sets folder search paths for referenced documents.
        """
        raise NotImplementedError

    def set_selection_filter(self):
        """
        Sets the selection filter for the specified item type.
        """
        raise NotImplementedError

    def set_selection_filters(self):
        """
        Sets the status for multiple selection filters.
        """
        raise NotImplementedError

    def set_third_party_popup_menu_state(self):
        """
        Shows or hides a third-party popup (shortcut) menu.
        """
        raise NotImplementedError

    def set_toolbar_dock2(self):
        """
        Sets the docking state of a toolbar.
        """
        raise NotImplementedError

    def set_toolbar_visibility(self):
        """
        Sets whether a specified toolbar is visible.
        """
        raise NotImplementedError

    def set_user_preference_double_value(self):
        """
        Sets a system default user preference value (double).
        """
        raise NotImplementedError

    def set_user_preference_integer_value(self):
        """
        Sets a system default user preference value (integer).
        """
        raise NotImplementedError

    def set_user_preference_string_list_value(self):
        """
        Sets the name of the DXF mapping files.
        """
        raise NotImplementedError

    def set_user_preference_string_value(self):
        """
        Sets a system default user preference value (string).
        """
        raise NotImplementedError

    def set_user_preference_toggle(self):
        """
        Sets a system default user preference toggle.
        """
        raise NotImplementedError

    def show_batch_save_to_3dexperience_dlg(self):
        """
        Opens a dialog to save files to 3DEXPERIENCE.
        """
        raise NotImplementedError

    def show_bubble_tooltip(self):
        """
        Displays a bubble tooltip and flashes a toolbar button.
        """
        raise NotImplementedError

    def show_bubble_tooltip_at2(self):
        """
        Displays a bubble ToolTip at the specified screen location.
        """
        raise NotImplementedError

    def show_help(self):
        """
        Displays the specified Help topic.
        """
        raise NotImplementedError

    def show_third_party_popup_menu(self):
        """
        Sets where to show a third-party pop-up (shortcut) menu.
        """
        raise NotImplementedError

    def show_toolbar2(self):
        """
        Displays a toolbar. (Obsolete, not superseded)
        """
        raise NotImplementedError

    def show_user_notification(self):
        """
        Shows a user notification for a SOLIDWORKS add-in.
        """
        raise NotImplementedError

    def solidworks_explorer(self):
        """
        Starts SOLIDWORKS Explorer application.
        """
        raise NotImplementedError

    def uninstall_quick_tip_guide(self):
        """
        Uninstalls your add-in's Quick Tips.
        """
        raise NotImplementedError

    def unload_addin(self):
        """
        Unloads the specified add-in from SOLIDWORKS.
        """
        raise NotImplementedError

    def upload_to_my_solidworks_settings(self):
        """
        Uploads specified SOLIDWORKS Desktop settings to SOLIDWORKS Connected.
        """
        raise NotImplementedError

    def version_history(self):
        """
        Gets a list of strings indicating the versions in which a model was saved.
        """
        raise NotImplementedError



"""
swTaskPaneTab_e Enumeration

Reference:
https://help.solidworks.com/2019/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swTaskPaneTab_e.html

Status: 🟢
"""

from enum import IntEnum


class SWTaskPaneTabE(IntEnum):
    SW_TASK_PANE_TAB_SOLIDWORKS_RESOURCES = 0  # SolidWorks Resources tab
    SW_TASK_PANE_TAB_DESIGN_LIBRARY = 1  # Design Library tab
    SW_TASK_PANE_TAB_FILE_EXPLORER = 2  # File Explorer tab
    SW_TASK_PANE_TAB_VIEW_PALETTE = 3  # View Palette tab
    SW_TASK_PANE_TAB_APPEARANCES = 4  # Appearances, Scenes, and Decals tab
    SW_TASK_PANE_TAB_CUSTOM_PROPERTIES = 5  # Custom Properties tab
    SW_TASK_PANE_TAB_SOLIDWORKS_ADDINS = 6  # SolidWorks Add-Ins tab



"""
PART TOOLS // OPEN PART
"""

from pathlib import Path
from typing import Tuple

from pyswx.api.sldworks.interfaces.i_document_specification import IDocumentSpecification
from pyswx.api.sldworks.interfaces.i_model_doc_2 import IModelDoc2
from pyswx.api.sldworks.interfaces.i_part_doc import IPartDoc
from pyswx.api.sldworks.interfaces.i_sldworks import ISldWorks
from pyswx.api.swconst.enumerations import SWDocumentTypesE
from pyswx.api.swconst.enumerations import SWRebuildOnActivationOptionsE
from pyswx.exceptions import DocumentError


def open_part(
    swx: ISldWorks,
    part_path: Path,
    document_specification: IDocumentSpecification | None = None,
) -> Tuple[IModelDoc2, IPartDoc]:
    """
    Open a SolidWorks part document.

    Args:
        swx (ISldWorks): The SolidWorks application instance.
        part_path (Path): The path to the SolidWorks part file to be exported.
        document_specification (IDocumentSpecification | None, optional): The document specification to use. Defaults to None.

    Returns:
        Tuple[IModelDoc2, IPartDoc]: A tuple containing the model document and part document.

    Remarks:
        The following document specification options are set:
        - document_type: SWDocumentTypesE.SW_DOC_PART
        - use_light_weight_default: True
        - light_weight: True
        - silent: True
        - ignore_hidden_components: True

    Raises:
        DocumentError: Raised if there is an error opening the document.
        ValueError: Raised if no active document is found.
        ValueError: Raised if the active document is not a part.
    """
    part_open_spec = swx.get_open_doc_spec(file_name=part_path)
    part_open_spec.document_type = SWDocumentTypesE.SW_DOC_PART
    part_open_spec.use_light_weight_default = True
    part_open_spec.light_weight = True
    part_open_spec.silent = True
    part_open_spec.ignore_hidden_components = True
    part_model = swx.open_doc7(specification=document_specification or part_open_spec)

    if part_open_spec.warning is not None:
        swx.logger.warning(part_open_spec.warning.name)

    if part_open_spec.error is not None:
        swx.logger.error(part_open_spec.error.name)
        raise DocumentError(part_open_spec.error.name)

    if part_model is None:
        raise ValueError("No active document found")

    model_type = part_model.get_type()
    if model_type != SWDocumentTypesE.SW_DOC_PART:
        raise ValueError(f"Active document is not a part: {model_type.name}")

    part_model = swx.activate_doc_3(
        name=part_model.get_path_name(),
        use_user_preferences=False,
        option=SWRebuildOnActivationOptionsE.SW_REBUILD_ACTIVE_DOC,
    )

    return part_model, IPartDoc(part_model.com_object)



"""
swDocumentTypes_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swDocumentTypes_e.html

Status: 🟢
"""

from enum import IntEnum


class SWDocumentTypesE(IntEnum):
    """Document types."""

    SW_DOC_NONE = 0  # No document
    SW_DOC_PART = 1  # Part document
    SW_DOC_ASSEMBLY = 2  # Assembly document
    SW_DOC_DRAWING = 3  # Drawing document
    SW_DOC_SDM = 4  # SDM document
    SW_DOC_LAYOUT = 5  # Layout document
    SW_DOC_IMPORTED_PART = 6  # Imported part (Multi-CAD)
    SW_DOC_IMPORTED_ASSEMBLY = 7  # Imported assembly (Multi-CAD)



"""
swAppNotify_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swAppNotify_e.html

Status: 🟢
"""

from enum import IntEnum


class SWAppNotifyE(IntEnum):
    """Application notifications."""

    SW_APP_FILE_OPEN_NOTIFY = 1  # Obsolete
    SW_APP_FILE_CLOSE_NOTIFY = 32  # FileCloseNotify
    SW_APP_FILE_NEW_NOTIFY = 2  # Obsolete
    SW_APP_FILE_NEW_NOTIFY2 = 12  # FileNewNotify2
    SW_APP_FILE_NEW_PRE_NOTIFY = 26  # FileNewPreNotify
    SW_APP_FILE_OPEN_NOTIFY2 = 13  # FileOpenNotify2
    SW_APP_FILE_OPEN_POST_NOTIFY = 22  # FileOpenPostNotify
    SW_APP_FILE_OPEN_PRE_NOTIFY = 21  # FileOpenPreNotify
    SW_APP_ACTIVE_DOC_CHANGE_NOTIFY = 4  # ActiveDocChangeNotify
    SW_APP_ACTIVE_MODEL_DOC_CHANGE_NOTIFY = 5  # ActiveModelDocChangeNotify
    SW_APP_BACKGROUND_PROCESSING_START_NOTIFY = 33  # BackgroundProcessingStartNotify
    SW_APP_BACKGROUND_PROCESSING_END_NOTIFY = 34  # BackgroundProcessingEndNotify
    SW_APP_BEGIN_3D_INTERCONNECT_TRANSLATION_NOTIFY = (
        37  # Begin3DInterconnectTranslationNotify
    )
    SW_APP_END_3D_INTERCONNECT_TRANSLATION_NOTIFY = (
        38  # End3DInterconnectTranslationNotify
    )
    SW_APP_BEGIN_RECORD_NOTIFY = 24  # Not used
    SW_APP_END_RECORD_NOTIFY = 25  # Not used
    SW_APP_BEGIN_TRANSLATION_NOTIFY = 16  # BeginTranslationNotify
    SW_APP_END_TRANSLATION_NOTIFY = 16  # EndTranslationNotify
    SW_APP_COMMAND_CLOSE_NOTIFY = 29  # CommandCloseNotify
    SW_APP_COMMAND_OPEN_PRE_NOTIFY = 31  # CommandOpenPreNotify
    SW_APP_DESTROY_NOTIFY = 3  # DestroyNotify
    SW_APP_DISPLAY_PANE_ACTIVATION_NOTIFY = 45  # DisplayPaneActivationNotify
    SW_APP_DOCUMENT_CONVERSION_NOTIFY = 9  # DocumentConversionNotify
    SW_APP_DOCUMENT_LOAD_NOTIFY = 27  # Obsolete
    SW_APP_DOCUMENT_LOAD_NOTIFY2 = 28  # DocumentLoadNotify2
    SW_APP_INTERFACE_BRIGHTNESS_THEME_CHANGE_NOTIFY = (
        35  # InterfaceBrightnessThemeChangeNotify
    )
    SW_APP_JOURNAL_WRITE_NOTIFY = 27  # Not used
    SW_APP_LIGHT_PM_CREATE_NOTIFY = -1  # Not used (no value given)
    SW_APP_LIGHT_SHEET_CREATE_NOTIFY = 18  # LightSheetCreateNotify
    SW_APP_LIGHTWEIGHT_COMPONENT_OPEN_NOTIFY = 10  # Not used
    SW_APP_NON_NATIVE_FILE_OPEN_NOTIFY = 7  # NonNativeFileOpenNotify
    SW_APP_ON_IDLE_NOTIFY = 20  # OnIdleNotify
    SW_APP_PROMPT_FOR_FILENAME_NOTIFY = 15  # PromptForFilenameNotify
    SW_APP_PROMPT_FOR_MULTIPLE_FILENAMES_NOTIFY = 30  # PromptForMultipleFileNamesNotify
    SW_APP_PROPERTY_SHEET_CREATE_NOTIFY = 6  # PropertySheetCreateNotify
    SW_APP_REFERENCED_FILE_PRE_NOTIFY = 23  # ReferencedFilePreNotify
    SW_APP_REFERENCED_FILE_PRE_NOTIFY2 = 36  # ReferencedFilePreNotify2
    SW_APP_REFERENCE_NOT_FOUND_NOTIFY = 14  # ReferenceNotFoundNotify
    SW_APP_STANDARDS_DATABASE_CHANGE_NOTIFY = 19  # Not used
    SW_APP_TASK_PANE_COLLAPSE_NOTIFY = 44  # TaskPaneCollapseNotify
    SW_APP_TASK_PANE_EXPAND_NOTIFY = 43  # TaskPaneExpandNotify
    SW_APP_TASK_PANE_HIDE_NOTIFY = 41  # TaskPaneHideNotify
    SW_APP_TASK_PANE_PINNED_NOTIFY = 39  # TaskPanePinnedNotify
    SW_APP_TASK_PANE_SHOW_NOTIFY = 42  # TaskPaneShowNotify
    SW_APP_TASK_PANE_UNPINNED_NOTIFY = 40  # TaskPaneUnpinnedNotify
